@@ -2,20 +2,25 @@
  *
  * redland.h - Redland RDF Application Framework public API
  *
- * $Id: librdf.h,v 1.43.2.1 2004/02/23 21:40:40 cmdjb Exp $
+ * $Id: librdf.h,v 1.48 2004/11/01 20:43:32 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
@@ -31,10 +36,14 @@ extern "C" {
 #include <stdio.h>
 
 #ifdef WIN32
-#  ifdef REDLAND_INTERNAL
-#    define REDLAND_API _declspec(dllexport)
+#  ifdef REDLAND_STATIC
+#    define REDLAND_API
 #  else
-#    define REDLAND_API _declspec(dllimport)
+#    ifdef REDLAND_INTERNAL
+#      define REDLAND_API _declspec(dllexport)
+#    else
+#      define REDLAND_API _declspec(dllimport)
+#    endif
 #  endif
 #else
 #  define REDLAND_API
@@ -75,6 +84,7 @@ typedef struct librdf_parser_s librdf_parser;
 typedef struct librdf_parser_factory_s librdf_parser_factory;
 typedef struct librdf_query_s librdf_query;
 typedef struct librdf_query_factory_s librdf_query_factory;
+typedef struct librdf_query_results_s librdf_query_results;
 typedef struct librdf_serializer_s librdf_serializer;
 typedef struct librdf_serializer_factory_s librdf_serializer_factory;
 
@@ -103,6 +113,7 @@ extern const unsigned int librdf_version_decimal;
 /* FIXME: Should be replaced with automatically pulled
  * definitions from the listed rdf_*.h header files.
  */
+#include <rdf_log.h>
 #include <rdf_init.h>
 #include <rdf_iterator.h>
 #include <rdf_uri.h>
