@@ -2,26 +2,37 @@
  *
  * rdf_hash.c - RDF Hash Implementation
  *
- * $Id: rdf_hash.c,v 1.74 2004/01/14 21:28:25 cmdjb Exp $
+ * $Id: rdf_hash.c,v 1.79 2004/11/01 20:43:33 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -33,7 +44,7 @@
 #include <stdlib.h> /* for strtol */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 
 #include <rdf_hash.h>
 #ifdef HAVE_BDB_HASH
@@ -206,12 +217,12 @@ librdf_hash_register_factory(librdf_world *world, const char *name,
   hash=(librdf_hash_factory*)LIBRDF_CALLOC(librdf_hash_factory, 1,
                                            sizeof(librdf_hash_factory));
   if(!hash)
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_HASH, "Out of memory");
   
   name_copy=(char*)LIBRDF_CALLOC(cstring, strlen(name)+1, 1);
   if(!name_copy) {
     LIBRDF_FREE(librdf_hash, hash);
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_HASH, "Out of memory");
   }
   strcpy(name_copy, name);
   hash->name=name_copy;
@@ -220,7 +231,8 @@ librdf_hash_register_factory(librdf_world *world, const char *name,
     if(!strcmp(h->name, name_copy)) {
       LIBRDF_FREE(cstring, name_copy); 
       LIBRDF_FREE(librdf_hash_factory, hash);
-      LIBRDF_ERROR2(world, "hash %s already registered", h->name);
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                 "hash %s already registered", h->name);
       return;
     }
   }
@@ -688,7 +700,9 @@ librdf_hash_get_all_iterator_get_method(void* iterator, int flags)
       break;
 
     default:
-      LIBRDF_ERROR2(context->hash->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(context->hash->world, 
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       result=NULL;
       break;
   }
@@ -1290,7 +1304,9 @@ librdf_hash_from_string (librdf_hash* hash, const char *string)
         break;
         
       default:
-        LIBRDF_ERROR2(hash->world, "No such state %d", state);
+        librdf_log(hash->world, 
+                   0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                   "No such state %d", state);
         return 1;
     }
   }
@@ -1314,7 +1330,9 @@ librdf_hash_from_array_of_strings (librdf_hash* hash, char **array)
   for(i=0; (key.data=array[i]); i+=2) {
     value.data=array[i+1];
     if(!value.data) {
-      LIBRDF_ERROR2(hash->world, "Array contains an odd number of strings - %d", i);
+      librdf_log(hash->world, 
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                 "Array contains an odd number of strings - %d", i);
       return 1;
     }
     key.size=strlen((char*)key.data);
