@@ -2,26 +2,37 @@
  *
  * rdf_model.c - RDF Model implementation
  *
- * $Id: rdf_model.c,v 1.78 2004/01/25 14:51:48 cmdjb Exp $
+ * $Id: rdf_model.c,v 1.97 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -29,7 +40,7 @@
 #include <stdlib.h> /* for exit()  */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 
 #ifndef STANDALONE
 /* prototypes for helper functions */
@@ -44,7 +55,7 @@ void
 librdf_init_model(librdf_world *world)
 {
   /* Always have model storage implementation available */
-  librdf_init_model_storage();
+  librdf_init_model_storage(world);
 }
 
 
@@ -82,16 +93,27 @@ librdf_delete_model_factories(void)
 }
 
 
+/*
+ * librdf_model_supports_contexts - helper function to determine if this model supports contexts
+ **/
+static inline int
+librdf_model_supports_contexts(librdf_model* model) {
+  return model->supports_contexts;
+}
+
+
+
 /* class methods */
 
 /**
  * librdf_model_register_factory - Register a model factory
+ * @world: redland world object
  * @name: the model factory name
  * @factory: pointer to function to call to register the factory
  * 
  **/
 void
-librdf_model_register_factory(const char *name,
+librdf_model_register_factory(librdf_world *world, const char *name,
                               void (*factory) (librdf_model_factory*)) 
 {
   librdf_model_factory *model, *h;
@@ -104,12 +126,12 @@ librdf_model_register_factory(const char *name,
   model=(librdf_model_factory*)LIBRDF_CALLOC(librdf_model_factory, 1,
                                              sizeof(librdf_model_factory));
   if(!model)
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_MODEL, "Out of memory");
 
   name_copy=(char*)LIBRDF_CALLOC(cstring, strlen(name)+1, 1);
   if(!name_copy) {
     LIBRDF_FREE(librdf_model, model);
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_MODEL, "Out of memory");
   }
   strcpy(name_copy, name);
   model->name=name_copy;
@@ -117,7 +139,9 @@ librdf_model_register_factory(const char *name,
   for(h = models; h; h = h->next ) {
     if(!strcmp(h->name, name_copy)) {
       LIBRDF_FREE(cstring, name_copy);
-      LIBRDF_ERROR2(model->world, "model %s already registered", h->name);
+      librdf_log(world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_MODEL, NULL,
+                 "model %s already registered", h->name);
       return;
     }
   }
@@ -223,6 +247,7 @@ librdf_new_model_with_options(librdf_world *world,
                               librdf_storage *storage, librdf_hash* options)
 {
   librdf_model *model;
+  librdf_uri *uri;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
 
@@ -249,6 +274,16 @@ librdf_new_model_with_options(librdf_world *world,
     return NULL;
   }
 
+  uri=librdf_new_uri(world, (const unsigned char*)LIBRDF_MODEL_FEATURE_CONTEXTS);
+  if(uri) {
+    librdf_node *node=librdf_model_get_feature(model, uri);
+    if(node) {
+      model->supports_contexts=atoi((const char*)librdf_node_get_literal_value(node));
+      librdf_free_node(node);
+    }
+    librdf_free_uri(uri);
+  }
+
   model->usage=1;
 
   return model;
@@ -272,6 +307,7 @@ librdf_new_model_from_model(librdf_model* model)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
 
   new_model=model->factory->clone(model);
+  new_model->supports_contexts=model->supports_contexts;
   new_model->usage=1;
   return new_model;
 }
@@ -356,6 +392,10 @@ librdf_model_size(librdf_model* model)
  * shared with the model.  It must be a complete statement - all
  * of subject, predicate, object parts must be present.
  *
+ * If the statement already exists in the model, it is not added.
+ * Duplicate statements can be added when used with Redland Contexts
+ * such as with &librdf_model_context_add_statement
+ *
  * Return value: non 0 on failure
  **/
 int
@@ -366,6 +406,10 @@ librdf_model_add_statement(librdf_model* model, librdf_statement* statement)
 
   if(!librdf_statement_is_complete(statement))
     return 1;
+
+  /* Do not add duplicate statements */
+  if(librdf_model_contains_statement(model, statement))
+    return 0;
   
   return model->factory->add_statement(model, statement);
 }
@@ -376,6 +420,10 @@ librdf_model_add_statement(librdf_model* model, librdf_statement* statement)
  * @model: model object
  * @statement_stream: stream of statements to use
  * 
+ * If any of the statements already exists in the store, they are not
+ * added unless Redland contexts are being used.  See also
+ * &librdf_model_context_add_statements
+ *
  * Return value: non 0 on failure
  **/
 int
@@ -437,7 +485,7 @@ librdf_model_add(librdf_model* model, librdf_node* subject,
  * @predicate: &librdf_node of predicate
  * @literal: string literal content
  * @xml_language: language of literal
- * @datatype: datatype librdf_uri
+ * @datatype_uri: datatype &librdf_uri
  * 
  * After this method, the &librdf_node subject and predicate become
  * owned by the model.
@@ -988,6 +1036,12 @@ librdf_model_context_add_statement(librdf_model* model,
   if(!librdf_statement_is_complete(statement))
     return 1;
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return 1;
+  }
+
   return model->factory->context_add_statement(model, context, statement);
 }
 
@@ -1016,6 +1070,12 @@ librdf_model_context_add_statements(librdf_model* model,
   if(!stream)
     return 1;
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return 1;
+  }
+
   if(model->factory->context_add_statements)
     return model->factory->context_add_statements(model, context, stream);
 
@@ -1058,6 +1118,12 @@ librdf_model_context_remove_statement(librdf_model* model,
   if(!librdf_statement_is_complete(statement))
     return 1;
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return 1;
+  }
+
   return model->factory->context_remove_statement(model, context, statement);
 }
 
@@ -1078,6 +1144,12 @@ librdf_model_context_remove_statements(librdf_model* model,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_node, 1);
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return 1;
+  }
+
   if(model->factory->context_remove_statements)
     return model->factory->context_remove_statements(model, context);
 
@@ -1110,6 +1182,12 @@ librdf_model_context_as_stream(librdf_model* model, librdf_node* context)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_node, NULL);
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return NULL;
+  }
+
   return model->factory->context_serialize(model, context);
 }
 
@@ -1130,62 +1208,33 @@ librdf_model_context_serialize(librdf_model* model, librdf_node* context)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_node, NULL);
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return NULL;
+  }
+
   return model->factory->context_serialize(model, context);
 }
 
 
 /**
- * librdf_model_query - Run a query against the model returning matching statements
+ * librdf_model_query_execute - Execute a query against the model
  * @model: &librdf_model object
  * @query: &librdf_query object
  * 
  * Run the given query against the model and return a &librdf_stream of
  * matching &librdf_statement objects
  * 
- * Return value: &librdf_stream of matching statements (may be empty) or NULL on failure
+ * Return value: &librdf_query_results or NULL on failure
  **/
-librdf_stream*
-librdf_model_query(librdf_model* model, librdf_query* query) 
+librdf_query_results*
+librdf_model_query_execute(librdf_model* model, librdf_query* query) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, librdf_query, NULL);
 
-  return model->factory->query(model, query);
-}
-
-
-/**
- * librdf_model_query_string - Run a query string against the model returning matching statements
- * @model: &librdf_model object
- * @name: query language name
- * @uri: query language URI (or NULL)
- * @query_string: string in query language
- * 
- * Run the given query against the model and return a &librdf_stream of
- * matching &librdf_statement objects
- * 
- * Return value: &librdf_stream of matching statements (may be empty) or NULL on failure
- **/
-librdf_stream*
-librdf_model_query_string(librdf_model* model,
-                          const char *name, librdf_uri *uri,
-                          const unsigned char *query_string)
-{
-  librdf_query *query;
-  librdf_stream *stream;
-
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_string, string, NULL);
-
-  query=librdf_new_query(model->world, name, uri, query_string);
-  if(!query)
-    return NULL;
-  
-  stream=librdf_model_query(model, query);
-
-  librdf_free_query(query);
-  
-  return stream;
+  return model->factory->query_execute(model, query);
 }
 
 
@@ -1247,14 +1296,25 @@ librdf_model_find_statements_in_context(librdf_model* model, librdf_statement* s
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, NULL);
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return NULL;
+  }
 
   if(model->factory->find_statements_in_context)
     return model->factory->find_statements_in_context(model, statement, context_node);
 
-  stream=librdf_model_context_as_stream(model, context_node);
-  if(!stream)
+  statement=librdf_new_statement_from_statement(statement);
+  if(!statement)
     return NULL;
 
+  stream=librdf_model_context_as_stream(model, context_node);
+  if(!stream) {
+    librdf_free_statement(statement);
+    return NULL;
+  }
+
   librdf_stream_add_map(stream,
                         &librdf_stream_statement_find_map,
                         (librdf_stream_map_free_context_handler)&librdf_free_statement, (void*)statement);
@@ -1277,6 +1337,12 @@ librdf_model_get_contexts(librdf_model* model)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
 
+  if(!librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return NULL;
+  }
+
   if(model->factory->get_contexts)
     return model->factory->get_contexts(model);
   else
@@ -1289,7 +1355,7 @@ librdf_model_get_contexts(librdf_model* model)
  * @model: &librdf_model object
  * @feature: &librdf_uri feature property
  * 
- * Return value: &librdf_node feature value or NULL if no such feature
+ * Return value: new &librdf_node feature value or NULL if no such feature
  * exists or the value is empty.
  **/
 librdf_node*
@@ -1349,6 +1415,12 @@ librdf_model_find_statements_with_options(librdf_model* model,
                                           librdf_node* context_node,
                                           librdf_hash* options) 
 {
+  if(context_node && !librdf_model_supports_contexts(model)) {
+    librdf_log(model->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_MODEL, NULL,
+               "Model does not support contexts");
+    return NULL;
+  }
+
   if(model->factory->find_statements_with_options)
     return model->factory->find_statements_with_options(model, statement, context_node, options);
   else
@@ -1356,6 +1428,110 @@ librdf_model_find_statements_with_options(librdf_model* model,
 }
 
 
+/**
+ * librdf_model_load - Load content from a URI into the model
+ * @model: &librdf_model object
+ * @uri: the URI to read the content
+ * @name: the name of the parser (or NULL)
+ * @mime_type: the MIME type of the syntax (NULL if not used)
+ * @type_uri: URI identifying the syntax (NULL if not used)
+ *
+ * If the name field is NULL, the library will try to guess
+ * the parser to use from the uri, mime_type and type_uri fields.
+ * This is done via the raptor_guess_parser_name function.
+ * 
+ * Return value: non 0 on failure
+ **/
+int
+librdf_model_load(librdf_model* model, librdf_uri *uri,
+                  const char *name, const char *mime_type, 
+                  librdf_uri *type_uri)
+{
+  int rc=0;
+  librdf_parser* parser;
+
+  if(name && !*name)
+    name=NULL;
+  if(mime_type && !*mime_type)
+    mime_type=NULL;
+
+  if(!name)
+    name=raptor_guess_parser_name((raptor_uri*)type_uri, mime_type,
+                                  NULL, 0, librdf_uri_as_string(uri));
+  parser=librdf_new_parser(model->world, name, NULL, NULL);
+  if(!parser)
+    return 1;
+
+  rc=librdf_parser_parse_into_model(parser, uri, NULL, model);
+  librdf_free_parser(parser);
+  return rc;
+}
+
+
+/**
+ * librdf_model_to_counted_string - Write serialized model to a string
+ * @model: &librdf_model object
+ * @uri: base URI to use in serializing (or NULL if not used)
+ * @name: the name of the serializer (or NULL for default)
+ * @mime_type: the MIME type of the syntax (NULL if not used)
+ * @type_uri: URI identifying the syntax (NULL if not used)
+ * @string_length_p: pointer to location to store string length (or NULL)
+ *
+ * If the name field is NULL, the default serializer will be used.
+ *
+ * Note: the returned string must be freed by the caller.
+ *
+ * Return value: new string or NULL on failure
+ **/
+unsigned char*
+librdf_model_to_counted_string(librdf_model* model, librdf_uri *uri,
+                               const char *name, const char *mime_type, 
+                               librdf_uri *type_uri, size_t* string_length_p)
+{
+  unsigned char *string=NULL;
+  librdf_serializer* serializer;
+  
+  if(name && !*name)
+    name=NULL;
+  if(mime_type && !*mime_type)
+    mime_type=NULL;
+
+  serializer=librdf_new_serializer(model->world, name, mime_type, type_uri);
+  if(!serializer)
+    return NULL;
+
+  string=librdf_serializer_serialize_model_to_counted_string(serializer,
+                                                             uri, model,
+                                                             string_length_p);
+  librdf_free_serializer(serializer);
+  return string;
+}
+
+
+/**
+ * librdf_model_to_string - Write serialized model to a string
+ * @model: &librdf_model object
+ * @uri: base URI to use in serializing (or NULL if not used)
+ * @name: the name of the serializer (or NULL for default)
+ * @mime_type: the MIME type of the syntax (NULL if not used)
+ * @type_uri: URI identifying the syntax (NULL if not used)
+ *
+ * If the name field is NULL, the default serializer will be used.
+ *
+ * Note: the returned string must be freed by the caller.
+ *
+ * Return value: new string or NULL on failure
+ **/
+unsigned char*
+librdf_model_to_string(librdf_model* model, librdf_uri *uri,
+                       const char *name, const char *mime_type, 
+                       librdf_uri *type_uri) {
+  return librdf_model_to_counted_string(model, uri, 
+                                        name, mime_type, type_uri,
+                                        NULL);
+}
+
+
 #endif
 
 
@@ -1399,7 +1575,7 @@ main(int argc, char *argv[])
   librdf_statement *statement;
   librdf_parser* parser;
   librdf_stream* stream;
-  const char *parser_name="raptor";
+  const char *parser_name="rdfxml";
   #define URI_STRING_COUNT 2
   const unsigned char *file_uri_strings[URI_STRING_COUNT]={(const unsigned char*)"http://example.org/test1.rdf", (const unsigned char*)"http://example.org/test2.rdf"};
   const unsigned char *file_content[URI_STRING_COUNT]={(const unsigned char*)EX1_CONTENT, (const unsigned char*)EX2_CONTENT};
@@ -1413,6 +1589,10 @@ main(int argc, char *argv[])
   librdf_node *n1, *n2;
   int count;
   int expected_count;
+#define EXPECTED_BAD_STRING_LENGTH 299
+  librdf_uri* base_uri;
+  unsigned char *string;
+  size_t string_length=0;
 
   librdf_world_open(world);
   
@@ -1614,6 +1794,20 @@ main(int argc, char *argv[])
     librdf_model_print(model, stderr);
   }
 
+  fprintf(stderr, "%s: Serializing model\n", program);
+  base_uri=librdf_new_uri(world, (const unsigned char*)"http://example.org/base#");
+  string=librdf_model_to_counted_string(model, base_uri,
+                                        "rdfxml", 
+                                        NULL, NULL,
+                                        &string_length);
+  if(string_length != EXPECTED_BAD_STRING_LENGTH) {
+    fprintf(stderr, "%s: Serialising to RDF/XML returned string size %d, expected %d\n", program,
+            string_length, EXPECTED_BAD_STRING_LENGTH);
+    return 1;
+  }
+  librdf_free_uri(base_uri);
+  free(string);
+  fprintf(stderr, "%s: Serialized OK\n", program);
 
   fprintf(stderr, "%s: Freeing URIs and Nodes\n", program);
   for (i=0; i<URI_STRING_COUNT; i++) {
