@@ -2,26 +2,37 @@
  *
  * rdf_model_storage.c - RDF Model Storage implementation
  *
- * $Id: rdf_model_storage.c,v 1.11 2004/01/25 16:18:37 cmdjb Exp $
+ * $Id: rdf_model_storage.c,v 1.18 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -29,7 +40,7 @@
 #include <stdlib.h> /* for exit()  */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 
 
 typedef struct 
@@ -457,34 +468,27 @@ librdf_model_storage_context_serialize(librdf_model* model,
 
 
 /**
- * librdf_model_storage_query - Run a query against the model returning matching statements
+ * librdf_model_storage_query_execute - Run a query against the model returning librdf_query_results
  * @model: &librdf_model object
  * @query: &librdf_query object
  * 
- * Run the given query against the model and return a &librdf_stream of
- * matching &librdf_statement objects
+ * Run the given query against the model and return a &librdf_query_results
  * 
- * Return value: &librdf_stream of matching statements (may be empty) or NULL on failure
+ * Return value: &librdf_query_results or NULL on failure
  **/
-static librdf_stream*
-librdf_model_storage_query(librdf_model* model, librdf_query* query) 
+static librdf_query_results*
+librdf_model_storage_query_execute(librdf_model* model,
+                                   librdf_query *query) 
 {
-  librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
-  librdf_stream *stream;
-  
-  librdf_query_open(query);
-
-  if(librdf_storage_supports_query(context->storage, query))
-    stream=librdf_storage_query(context->storage, query);
+  librdf_model_storage_context *mcontext=(librdf_model_storage_context *)model->context;
+  if(librdf_storage_supports_query(mcontext->storage, query))
+    return librdf_storage_query_execute(mcontext->storage, query);
   else
-    stream=librdf_query_run(query,model);
-
-  librdf_query_close(query);
-
-  return stream;
+    return librdf_query_execute(query,model);
 }
 
 
+
 /**
  * librdf_model_storage_sync - Synchronise the model to the storage
  * @model: &librdf_model object
@@ -534,7 +538,7 @@ librdf_model_storage_get_contexts(librdf_model* model)
  * @storage: &librdf_storage object
  * @feature: &librdf_uri feature property
  * 
- * Return value: &librdf_node feature value or NULL if no such feature
+ * Return value: new &librdf_node feature value or NULL if no such feature
  * exists or the value is empty.
  **/
 static librdf_node*
@@ -609,7 +613,7 @@ librdf_model_storage_register_factory(librdf_model_factory *factory)
   factory->context_serialize        = librdf_model_storage_context_serialize;
 
 
-  factory->query              = librdf_model_storage_query;
+  factory->query_execute      = librdf_model_storage_query_execute;
   factory->sync               = librdf_model_storage_sync;
   factory->get_storage        = librdf_model_storage_get_storage;
   factory->get_contexts       = librdf_model_storage_get_contexts;
@@ -620,8 +624,8 @@ librdf_model_storage_register_factory(librdf_model_factory *factory)
 
 
 void
-librdf_init_model_storage(void)
+librdf_init_model_storage(librdf_world *world)
 {
-  librdf_model_register_factory("storage", 
+  librdf_model_register_factory(world, "storage", 
                                 &librdf_model_storage_register_factory);
 }
