@@ -2,26 +2,37 @@
  *
  * rdf_node.c - RDF Node (Resource / Literal) Implementation
  *
- * $Id: rdf_node.c,v 1.91 2004/01/14 21:28:39 cmdjb Exp $
+ * $Id: rdf_node.c,v 1.101 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -29,7 +40,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 /* needed for utf8 functions and definition of 'byte' */
 #include <rdf_utf8.h>
 
@@ -62,10 +73,10 @@ librdf_init_node(librdf_world* world)
   for(i=0; i<H_COUNT; i++) {
     world->nodes_hash[i]=librdf_new_hash(world, NULL);
     if(!world->nodes_hash[i])
-      LIBRDF_FATAL1(world, "Failed to create Nodes hash from factory");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_NODE, "Failed to create Nodes hash from factory");
     
     if(librdf_hash_open(world->nodes_hash[i], NULL, 0, 1, 1, NULL))
-      LIBRDF_FATAL1(world, "Failed to open Nodes hash");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_NODE, "Failed to open Nodes hash");
   }
 }
 
@@ -338,7 +349,7 @@ librdf_new_node_from_literal(librdf_world *world,
 /**
  * librdf_new_node_from_typed_literal -  Constructor - create a new typed literal librdf_node object
  * @world: redland world object
- * @string: literal string value
+ * @value: literal string value
  * @xml_language: literal XML language (or NULL, empty string)
  * @datatype_uri: URI of typed literal datatype or NULL
  * 
@@ -624,7 +635,7 @@ librdf_free_node(librdf_node *node)
       key.data=&node->value.resource.uri;
       key.size=sizeof(librdf_uri*);
       if(librdf_hash_delete_all(node->world->nodes_hash[H_RESOURCE], &key) )
-        LIBRDF_FATAL1(world, "Hash deletion failed");
+        LIBRDF_FATAL1(node->world, LIBRDF_FROM_NODE, "Hash deletion failed");
 
       librdf_free_uri(node->value.resource.uri);
       break;
@@ -634,7 +645,7 @@ librdf_free_node(librdf_node *node)
         key.data=node->value.literal.key;
         key.size=node->value.literal.size;
         if(librdf_hash_delete_all(node->world->nodes_hash[H_LITERAL], &key) )
-          LIBRDF_FATAL1(world, "Hash deletion failed");
+          LIBRDF_FATAL1(node->world, LIBRDF_FROM_NODE, "Hash deletion failed");
         LIBRDF_FREE(cstring, node->value.literal.key);
       }
       
@@ -650,7 +661,7 @@ librdf_free_node(librdf_node *node)
       key.data=node->value.blank.identifier;
       key.size=node->value.blank.identifier_len;
       if(librdf_hash_delete_all(node->world->nodes_hash[H_BLANK], &key) )
-        LIBRDF_FATAL1(world, "Hash deletion failed");
+        LIBRDF_FATAL1(node->world, LIBRDF_FROM_NODE, "Hash deletion failed");
 
       if(node->value.blank.identifier != NULL)
         LIBRDF_FREE(cstring, node->value.blank.identifier);
@@ -882,7 +893,7 @@ librdf_node_get_li_ordinal(librdf_node* node) {
   LIBRDF_ASSERT_RETURN((node->type != LIBRDF_NODE_TYPE_RESOURCE),
                        "node is not type resource", 0);
 
-  if(node->type != LIBRDF_NODE_TYPE_RESOURCE);
+  if(node->type != LIBRDF_NODE_TYPE_RESOURCE)
     return -1;
 
   uri_string=librdf_uri_as_string(node->value.resource.uri); 
@@ -904,7 +915,7 @@ unsigned char *
 librdf_node_get_blank_identifier(librdf_node* node) {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(node, librdf_node, NULL);
   LIBRDF_ASSERT_RETURN((node->type != LIBRDF_NODE_TYPE_BLANK),
-                       "node is not type resource", NULL);
+                       "node is not type blank", NULL);
 
   return node->value.blank.identifier;
 }
@@ -981,9 +992,9 @@ librdf_node_to_string(librdf_node* node)
 unsigned char*
 librdf_node_to_counted_string(librdf_node* node, size_t* len_p) 
 {
-  unsigned char *uri_string;
-  size_t len;
-  unsigned char *s;
+  unsigned char *uri_string, *datatype_uri_string;
+  size_t len, datatype_len, language_len;
+  unsigned char *s, *d;
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(node, librdf_node, NULL);
 
@@ -1005,13 +1016,39 @@ librdf_node_to_counted_string(librdf_node* node, size_t* len_p)
     break;
   case LIBRDF_NODE_TYPE_LITERAL:
     len=node->value.literal.string_len;
+    if(node->value.literal.xml_language) {
+      language_len=strlen(node->value.literal.xml_language);
+      len+=1+language_len;
+    }
+    
+    if(node->value.literal.datatype_uri) {
+      datatype_uri_string=librdf_uri_to_counted_string(node->value.literal.datatype_uri, &datatype_len);
+      len+=4+datatype_len;
+    } else
+      datatype_uri_string=NULL;
     if(len_p)
       *len_p=len;
     s=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
     if(!s)
       return NULL;
     /* use strcpy here to add \0 to end of literal string */
-    strcpy((char*)s, (const char*)node->value.literal.string);
+    d=s;
+    strncpy((char*)d, (const char*)node->value.literal.string, node->value.literal.string_len);
+    d+= node->value.literal.string_len;
+    
+    if(node->value.literal.xml_language) {
+      *d++='@';
+      strncpy((char*)d, node->value.literal.xml_language, language_len);
+      d+= language_len;
+    }
+    if(datatype_uri_string) {
+      strncpy((char*)d, "^^<", 3);
+      d+= 3;
+      strncpy((char*)d, (const char*)datatype_uri_string, datatype_len);
+      d+= datatype_len;
+      *d++='>';
+    }
+    *d='\0';
     break;
   case LIBRDF_NODE_TYPE_BLANK:
     len=node->value.blank.identifier_len + 2;
@@ -1023,7 +1060,9 @@ librdf_node_to_counted_string(librdf_node* node, size_t* len_p)
     sprintf((char*)s, "(%s)", node->value.blank.identifier);
     break;
   default:
-    LIBRDF_ERROR2(node->world, "Do not know how to print node type %d\n", node->type);
+      librdf_log(node->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
+                 "Do not know how to print node type %d\n", node->type);
     return NULL;
   }
   return s;
@@ -1092,7 +1131,9 @@ librdf_node_get_digest(librdf_node* node)
       librdf_digest_final(d);
       break;
     default:
-      LIBRDF_ERROR2(world, "Do not know how to make digest for node type %d\n", node->type);
+      librdf_log(world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
+                 "Do not know how to make digest for node type %d\n", node->type);
       return NULL;
   }
   
@@ -1248,7 +1289,9 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
       break;
       
     default:
-      LIBRDF_ERROR2(node->world, "Do not know how to encode node type %d\n", node->type);
+      librdf_log(node->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
+                 "Do not know how to encode node type %d\n", node->type);
       return 0;
   }
   
@@ -1371,7 +1414,9 @@ librdf_node_decode(librdf_world *world,
     break;
 
   default:
-    LIBRDF_ERROR2(world, "Illegal node encoding '%c' seen\n", buffer[0]);
+      librdf_log(world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
+                 "Illegal node encoding '%c' seen\n", buffer[0]);
     return NULL;
   }
   
@@ -1435,7 +1480,9 @@ librdf_node_static_iterator_get_method(void* iterator, int flags)
       return NULL;
 
     default:
-      LIBRDF_ERROR2(context->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(context->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       return NULL;
   }
 }
