@@ -2,26 +2,37 @@
  *
  * rdf_parser.c - RDF Parser Factory implementation
  *
- * $Id: rdf_parser.c,v 1.45 2004/01/14 21:28:42 cmdjb Exp $
+ * $Id: rdf_parser.c,v 1.57 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -29,7 +40,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 
 
 #ifndef STANDALONE
@@ -90,12 +101,12 @@ librdf_parser_register_factory(librdf_world *world,
   parser_factory=(librdf_parser_factory*)LIBRDF_CALLOC(librdf_parser_factory, 1,
 					       sizeof(librdf_parser_factory));
   if(!parser_factory)
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
   
   name_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(name)+1);
   if(!name_copy) {
     librdf_free_parser_factory(parser_factory);
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
   }
   strcpy(name_copy, name);
   parser_factory->name=name_copy;
@@ -106,7 +117,7 @@ librdf_parser_register_factory(librdf_world *world,
     mime_type_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(mime_type)+1);
     if(!mime_type_copy) {
       librdf_free_parser_factory(parser_factory);
-      LIBRDF_FATAL1(world, "Out of memory");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
     }
     strcpy(mime_type_copy, mime_type);
     parser_factory->mime_type=mime_type_copy;
@@ -119,7 +130,7 @@ librdf_parser_register_factory(librdf_world *world,
     uri=librdf_new_uri(world, uri_string);
     if(!uri) {
       librdf_free_parser_factory(parser_factory);
-      LIBRDF_FATAL1(world, "Out of memory");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
     }
     parser_factory->type_uri=uri;
   }
@@ -319,7 +330,7 @@ librdf_parser_parse_as_stream(librdf_parser* parser, librdf_uri* uri,
  * librdf_parser_parse_into_model - Parse a URI of content into an librdf_model
  * @parser: the parser
  * @uri: the URI to read the content
- * @base_uri: the base URI to use (or NULL if the same)
+ * @base_uri: the base URI to use (or NULL if the same as @uri)
  * @model: the model to use
  * 
  * Return value: non 0 on failure
@@ -397,7 +408,66 @@ librdf_parser_parse_string_into_model(librdf_parser* parser,
 }
 
 
+/**
+ * librdf_parser_parse_counted_string_as_stream - Parse a counted string of content to a librdf_stream of statements
+ * @parser: the parser
+ * @string: the string to parse
+ * @length: length of the string content (must be >0)
+ * @base_uri: the base URI to use
+ * 
+ * Return value: &librdf_stream of statements or NULL
+ **/
+librdf_stream*
+librdf_parser_parse_counted_string_as_stream(librdf_parser* parser, 
+                                             const unsigned char *string,
+                                             size_t length,
+                                             librdf_uri* base_uri)
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(parser, librdf_parser, NULL);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, NULL);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, NULL);
+  LIBRDF_ASSERT_RETURN(length < 1, "string length is not greater than zero", 
+                       NULL);
 
+  if(parser->factory->parse_counted_string_as_stream)
+    return parser->factory->parse_counted_string_as_stream(parser->context,
+                                                           string, length,
+                                                           base_uri);
+
+  return NULL;
+}
+
+
+/**
+ * librdf_parser_parse_counted_string_into_model - Parse a counted string of content into an librdf_model
+ * @parser: the parser
+ * @string: the content to parse
+ * @length: length of content (must be >0)
+ * @base_uri: the base URI to use
+ * @model: the model to use
+ * 
+ * Return value: non 0 on failure
+ **/
+int
+librdf_parser_parse_counted_string_into_model(librdf_parser* parser, 
+                                              const unsigned char *string,
+                                              size_t length,
+                                              librdf_uri* base_uri, 
+                                              librdf_model* model)
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(parser, librdf_parser, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
+  LIBRDF_ASSERT_RETURN(length < 1, "string length is not greater than zero", 1);
+
+  if(parser->factory->parse_counted_string_into_model)
+    return parser->factory->parse_counted_string_into_model(parser->context,
+                                                            string, length,
+                                                            base_uri, model);
+  
+  return 1;
+}
 
 
 /**
@@ -427,70 +497,19 @@ librdf_finish_parser(librdf_world *world)
 }
 
 
-/*
- * librdf_parser_error - Error from a parser - Internal
- **/
-void
-librdf_parser_error(librdf_parser* parser, const char *message, ...)
-{
-  va_list arguments;
-
-  if(parser->error_fn) {
-    parser->error_fn(parser->error_user_data, message);
-    return;
-  }
-  
-  va_start(arguments, message);
-
-  fprintf(stderr, "%s parser error - ", parser->factory->name);
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-
-  va_end(arguments);
-}
-
-
-/*
- * librdf_parser_warning - Warning from a parser - Internal
- **/
-void
-librdf_parser_warning(librdf_parser* parser, const char *message, ...)
-{
-  va_list arguments;
-
-  if(parser->warning_fn) {
-    parser->warning_fn(parser->warning_user_data, message);
-    return;
-  }
-  
-  va_start(arguments, message);
-
-  fprintf(stderr, "%s parser warning - ", parser->factory->name);
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-
-  va_end(arguments);
-}
-
-
 /**
  * librdf_parser_set_error - Set the parser error handling function
  * @parser: the parser
  * @user_data: user data to pass to function
  * @error_fn: pointer to the function
  * 
- * The function will receive callbacks when the parser fails.
+ * DEPRECATED: Does nothing
  * 
  **/
 void
 librdf_parser_set_error(librdf_parser* parser, void *user_data,
                         void (*error_fn)(void *user_data, const char *msg, ...))
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(parser, librdf_parser);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(error_fn, error handler);
-
-  parser->error_user_data=user_data;
-  parser->error_fn=error_fn;
 }
 
 
@@ -500,18 +519,13 @@ librdf_parser_set_error(librdf_parser* parser, void *user_data,
  * @user_data: user data to pass to function
  * @warning_fn: pointer to the function
  * 
- * The function will receive callbacks when the parser gives a warning.
- * 
+ * DEPRECATED: Does nothing.
+ *
  **/
 void
 librdf_parser_set_warning(librdf_parser* parser, void *user_data,
                           void (*warning_fn)(void *user_data, const char *msg, ...))
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(parser, librdf_parser);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(warning_fn, warning handler);
-
-  parser->warning_user_data=user_data;
-  parser->warning_fn=warning_fn;
 }
 
 
@@ -520,7 +534,7 @@ librdf_parser_set_warning(librdf_parser* parser, void *user_data,
  * @parser: &librdf_parser object
  * @feature: &librdf_Uuri feature property
  * 
- * Return value: the value of the feature or NULL if no such feature
+ * Return value: new &librdf_node feature value or NULL if no such feature
  * exists or the value is empty.
  **/
 librdf_node*
@@ -570,48 +584,172 @@ librdf_parser_set_feature(librdf_parser* parser, librdf_uri* feature,
 int main(int argc, char *argv[]);
 
 
+#define RDFXML_CONTENT \
+"<?xml version=\"1.0\"?>\n" \
+"<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" \
+"         xmlns:dc=\"http://purl.org/dc/elements/1.1/\">\n" \
+"  <rdf:Description rdf:about=\"http://purl.org/net/dajobe/\">\n" \
+"    <dc:title>Dave Beckett's Home Page</dc:title>\n" \
+"    <dc:creator>Dave Beckett</dc:creator>\n" \
+"    <dc:description>The generic home page of Dave Beckett.</dc:description>\n" \
+"  </rdf:Description>\n" \
+"</rdf:RDF>"
+
+#define NTRIPLES_CONTENT \
+"<http://purl.org/net/dajobe/> <http://purl.org/dc/elements/1.1/creator> \"Dave Beckett\" .\n" \
+"<http://purl.org/net/dajobe/> <http://purl.org/dc/elements/1.1/description> \"The generic home page of Dave Beckett.\" .\n" \
+"<http://purl.org/net/dajobe/> <http://purl.org/dc/elements/1.1/title> \"Dave Beckett's Home Page\" . \n"
+
+
+#define TURTLE_CONTENT \
+"@prefix dc: <http://purl.org/dc/elements/1.1/> .\n" \
+"\n" \
+"<http://purl.org/net/dajobe/>\n" \
+"      dc:creator \"Dave Beckett\" ;\n" \
+"      dc:description \"The generic home page of Dave Beckett.\" ; \n" \
+"      dc:title \"Dave Beckett's Home Page\" . \n"
+
+/* All the examples above give the same three triples */
+#define EXPECTED_TRIPLES_COUNT 3
+
+
 int
 main(int argc, char *argv[]) 
 {
-  librdf_parser* d;
-  char *test_parser_types[]={"ntriples", "raptor", NULL};
+  char *test_parser_types[]={"rdfxml", "ntriples", "turtle", NULL};
+  #define URI_STRING_COUNT 3
+  const unsigned char *file_uri_strings[URI_STRING_COUNT]={(const unsigned char*)"http://example.org/test1.rdf", (const unsigned char*)"http://example.org/test2.nt", (const unsigned char*)"http://example.org/test3.ttl"};
+  const unsigned char *file_content[URI_STRING_COUNT]={(const unsigned char*)RDFXML_CONTENT, (const unsigned char*)NTRIPLES_CONTENT, (const unsigned char*)TURTLE_CONTENT};
+  librdf_uri* uris[URI_STRING_COUNT];
   int i;
   char *type;
   char *program=argv[0];
   librdf_world *world;
 
   world=librdf_new_world();
-  librdf_world_init_mutex(world);
+  librdf_world_open(world);
 
-  /* Needed for URI use when registering factories */
-  librdf_init_digest(world);
-  librdf_init_hash(world);
-  librdf_init_uri(world);
+  for (i=0; i<URI_STRING_COUNT; i++) {
+    uris[i]=librdf_new_uri(world, file_uri_strings[i]);
+  }
 
-  /* initialise parser module */
-  librdf_init_parser(world);
-  
   for(i=0; (type=test_parser_types[i]); i++) {
-    fprintf(stderr, "%s: Trying to create new %s parser\n", program, type);
-    d=librdf_new_parser(world, type, NULL, NULL);
-    if(!d) {
+    librdf_storage* storage;
+    librdf_model *model;
+    librdf_parser* parser;
+    librdf_stream *stream=NULL;
+    size_t length=strlen((const char*)file_content[i]);
+    int size;
+
+    fprintf(stderr, "%s: Testing parsing syntax '%s'\n", program, type);
+
+    fprintf(stderr, "%s: Creating storage and model\n", program);
+    storage=librdf_new_storage(world, NULL, NULL, NULL);
+    if(!storage) {
+      fprintf(stderr, "%s: Failed to create new storage\n", program);
+      return(1);
+    }
+    model=librdf_new_model(world, storage, NULL);
+    if(!model) {
+      fprintf(stderr, "%s: Failed to create new model\n", program);
+      return(1);
+    }
+
+    fprintf(stderr, "%s: Creating %s parser\n", program, type);
+    parser=librdf_new_parser(world, type, NULL, NULL);
+    if(!parser) {
       fprintf(stderr, "%s: Failed to create new parser type %s\n", program, type);
       continue;
     }
     
-    fprintf(stderr, "%s: Freeing parser\n", program);
-    librdf_free_parser(d);
+
+    fprintf(stderr, "%s: Adding %s counted string content as stream\n", 
+            program, type);
+    if(!(stream=librdf_parser_parse_counted_string_as_stream(parser, 
+                                                             file_content[i],
+                                                             length,
+                                                             uris[i]))) {
+      fprintf(stderr, "%s: Failed to parse RDF from counted string %d as stream\n", program, i);
+      exit(1);
+    }
+    librdf_model_add_statements(model, stream);
+    librdf_free_stream(stream);
+
+    size=librdf_model_size(model);
+    fprintf(stderr, "%s: Model size is %d triples\n", program, size);
+    if(size != EXPECTED_TRIPLES_COUNT) {
+      fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
+              program, size, EXPECTED_TRIPLES_COUNT);
+      exit(1);
+    }
+
+    fprintf(stderr, "%s: Adding %s string content as stream\n", program, type);
+    if(!(stream=librdf_parser_parse_string_as_stream(parser, 
+                                                     file_content[i],
+                                                     uris[i]))) {
+      fprintf(stderr, "%s: Failed to parse RDF from string %d as stream\n", program, i);
+      exit(1);
+    }
+    librdf_model_add_statements(model, stream);
+    librdf_free_stream(stream);
+    
+    size=librdf_model_size(model);
+    fprintf(stderr, "%s: Model size is %d triples\n", program, size);
+    if(size != EXPECTED_TRIPLES_COUNT) {
+      fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
+              program, size, EXPECTED_TRIPLES_COUNT);
+      exit(1);
+    }
+    
+
+    fprintf(stderr, "%s: Adding %s counted string content\n", program, type);
+    if(librdf_parser_parse_counted_string_into_model(parser, 
+                                                     file_content[i],
+                                                     length,
+                                                     uris[i], model)) {
+      fprintf(stderr, "%s: Failed to parse RDF from counted string %d into model\n", program, i);
+      exit(1);
+    }
+    
+    size=librdf_model_size(model);
+    fprintf(stderr, "%s: Model size is %d triples\n", program, size);
+    if(size != EXPECTED_TRIPLES_COUNT) {
+      fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
+              program, size, EXPECTED_TRIPLES_COUNT);
+      exit(1);
+    }
+
+    
+    fprintf(stderr, "%s: Adding %s string content\n", program, type);
+    if(librdf_parser_parse_string_into_model(parser, 
+                                             file_content[i],
+                                             uris[i], model)) {
+      fprintf(stderr, "%s: Failed to parse RDF from string %d into model\n", program, i);
+      exit(1);
+    }
+
+    size=librdf_model_size(model);
+    fprintf(stderr, "%s: Model size is %d triples\n", program, size);
+    if(size != EXPECTED_TRIPLES_COUNT) {
+      fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
+              program, size, EXPECTED_TRIPLES_COUNT);
+      exit(1);
+    }
+
+    
+    fprintf(stderr, "%s: Freeing parser, model and storage\n", program);
+    librdf_free_parser(parser);
+    librdf_free_model(model);
+    librdf_free_storage(storage);
   }
   
+
+  fprintf(stderr, "%s: Freeing URIs\n", program);
+  for (i=0; i<URI_STRING_COUNT; i++) {
+    librdf_free_uri(uris[i]);
+  }
   
-  /* finish parser module */
-  librdf_finish_parser(world);
-
-  librdf_finish_uri(world);
-  librdf_finish_hash(world);
-  librdf_finish_digest(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
