@@ -2,26 +2,37 @@
  *
  * rdf_serializer.c - RDF Serializer Factory implementation
  *
- * $Id: rdf_serializer.c,v 1.22 2004/01/16 23:59:53 cmdjb Exp $
+ * $Id: rdf_serializer.c,v 1.36 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2002-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -29,7 +40,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 
 
 #ifndef STANDALONE
@@ -90,12 +101,12 @@ librdf_serializer_register_factory(librdf_world *world,
   serializer_factory=(librdf_serializer_factory*)LIBRDF_CALLOC(librdf_serializer_factory, 1,
 					       sizeof(librdf_serializer_factory));
   if(!serializer_factory)
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
   
   name_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(name)+1);
   if(!name_copy) {
     librdf_free_serializer_factory(serializer_factory);
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
   }
   strcpy(name_copy, name);
   serializer_factory->name=name_copy;
@@ -106,7 +117,7 @@ librdf_serializer_register_factory(librdf_world *world,
     mime_type_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(mime_type)+1);
     if(!mime_type_copy) {
       librdf_free_serializer_factory(serializer_factory);
-      LIBRDF_FATAL1(world, "Out of memory");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
     }
     strcpy(mime_type_copy, mime_type);
     serializer_factory->mime_type=mime_type_copy;
@@ -119,7 +130,7 @@ librdf_serializer_register_factory(librdf_world *world,
     uri=librdf_new_uri(world, uri_string);
     if(!uri) {
       librdf_free_serializer_factory(serializer_factory);
-      LIBRDF_FATAL1(world, "Out of memory");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
     }
     serializer_factory->type_uri=uri;
   }
@@ -285,12 +296,14 @@ librdf_free_serializer(librdf_serializer *serializer)
 /* methods */
 
 /**
- * librdf_serializer_serialize_model - Turn a librdf_model into a serialized form
+ * librdf_serializer_serialize_model - Write a serialized librdf_mode to a FILE*
  * @serializer: the serializer
  * @handle: file handle to serialize to
- * @base_uri: the base URI to use or NULL
+ * @base_uri: the base URI to use (or NULL)
  * @model: the &librdf_model model to use
  * 
+ * DEPRECATED: Use librdf_serializer_serialize_model_to_file_handle
+ *
  * Return value: non 0 on failure
  **/
 int
@@ -298,19 +311,40 @@ librdf_serializer_serialize_model(librdf_serializer* serializer,
                                   FILE *handle, librdf_uri* base_uri,
                                   librdf_model* model) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
-
-  return serializer->factory->serialize_model(serializer->context,
-                                              handle, base_uri, model);
+  return librdf_serializer_serialize_model_to_file_handle(serializer,
+                                                          handle, base_uri,
+                                                          model);
 }
 
 
 /**
- * librdf_serializer_serialize_model - Write a librdf_model into a serialized form to a file
+ * librdf_serializer_serialize_model_to_file_handle - Write a serialized librdf_model to a FILE*
+ * @serializer: the serializer
+ * @handle: file handle to serialize to
+ * @base_uri: the base URI to use (or NULL)
+ * @model: the &librdf_model model to use
+ * 
+ * Return value: non 0 on failure
+ **/
+int
+librdf_serializer_serialize_model_to_file_handle(librdf_serializer* serializer,
+                                                 FILE *handle, 
+                                                 librdf_uri* base_uri,
+                                                 librdf_model* model) 
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
+
+  return serializer->factory->serialize_model_to_file_handle(serializer->context,
+                                                             handle, base_uri, model);
+}
+
+
+/**
+ * librdf_serializer_serialize_model_to_file - Write a serialized librdf_model to a file
  * @serializer: the serializer
  * @name: filename to serialize to
- * @base_uri: the base URI to use or NULL
+ * @base_uri: the base URI to use (or NULL)
  * @model: the &librdf_model model to use
  * 
  * Return value: non 0 on failure
@@ -331,12 +365,62 @@ librdf_serializer_serialize_model_to_file(librdf_serializer* serializer,
   fh=fopen(name, "w+");
   if(!fh)
     return 1;
-  status=librdf_serializer_serialize_model(serializer, fh, base_uri, model);
+  status=librdf_serializer_serialize_model_to_file_handle(serializer, fh, 
+                                                          base_uri, model);
   fclose(fh);
   return status;
 }
 
 
+/**
+ * librdf_serializer_serialize_model_to_counted_string - Write a serialized librdf_model to a counted string
+ * @serializer: the serializer
+ * @base_uri: the base URI to use (or NULL)
+ * @model: the &librdf_model model to use
+ * @length_p: pointer to store length or NULL
+ * 
+ * Return value: non 0 on failure
+ **/
+unsigned char*
+librdf_serializer_serialize_model_to_counted_string(librdf_serializer* serializer,
+                                                    librdf_uri* base_uri,
+                                                    librdf_model* model,
+                                                    size_t* length_p) 
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, NULL);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
+
+  if(length_p)
+    *length_p=0;
+  
+  return serializer->factory->serialize_model_to_counted_string(serializer->context,
+                                                                base_uri, model,
+                                                                length_p);
+}
+
+
+/**
+ * librdf_serializer_serialize_model_to_string - Write a serialized librdf_model to a string
+ * @serializer: the serializer
+ * @base_uri: the base URI to use (or NULL)
+ * @model: the &librdf_model model to use
+ * 
+ * Return value: non 0 on failure
+ **/
+unsigned char*
+librdf_serializer_serialize_model_to_string(librdf_serializer* serializer,
+                                            librdf_uri* base_uri,
+                                            librdf_model* model) 
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, NULL);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, NULL);
+
+  return serializer->factory->serialize_model_to_counted_string(serializer->context,
+                                                                base_uri, model,
+                                                                NULL);
+}
+
+
 /**
  * librdf_init_serializer - Initialise the librdf_serializer class
  * @world: redland world object
@@ -345,7 +429,6 @@ void
 librdf_init_serializer(librdf_world *world) 
 {
   librdf_serializer_raptor_constructor(world);
-  librdf_serializer_rdfxml_constructor(world);
 }
 
 
@@ -360,70 +443,19 @@ librdf_finish_serializer(librdf_world *world)
 }
 
 
-/*
- * librdf_serializer_error - Error from a serializer - Internal
- **/
-void
-librdf_serializer_error(librdf_serializer* serializer, const char *message, ...)
-{
-  va_list arguments;
-
-  if(serializer->error_fn) {
-    serializer->error_fn(serializer->error_user_data, message);
-    return;
-  }
-  
-  va_start(arguments, message);
-
-  fprintf(stderr, "%s serializer error - ", serializer->factory->name);
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-
-  va_end(arguments);
-}
-
-
-/*
- * librdf_serializer_warning - Warning from a serializer - Internal
- **/
-void
-librdf_serializer_warning(librdf_serializer* serializer, const char *message, ...)
-{
-  va_list arguments;
-
-  if(serializer->warning_fn) {
-    serializer->warning_fn(serializer->warning_user_data, message);
-    return;
-  }
-  
-  va_start(arguments, message);
-
-  fprintf(stderr, "%s serializer warning - ", serializer->factory->name);
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-
-  va_end(arguments);
-}
-
-
 /**
  * librdf_serializer_set_error - Set the serializer error handling function
  * @serializer: the serializer
  * @user_data: user data to pass to function
  * @error_fn: pointer to the function
  * 
- * The function will receive callbacks when the serializer fails.
+ * DEPRECATED: Does nothing
  * 
  **/
 void
 librdf_serializer_set_error(librdf_serializer* serializer, void *user_data,
                             void (*error_fn)(void *user_data, const char *msg, ...))
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(serializer, librdf_serializer);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(error_fn, error handler);
-
-  serializer->error_user_data=user_data;
-  serializer->error_fn=error_fn;
 }
 
 
@@ -433,18 +465,13 @@ librdf_serializer_set_error(librdf_serializer* serializer, void *user_data,
  * @user_data: user data to pass to function
  * @warning_fn: pointer to the function
  * 
- * The function will receive callbacks when the serializer gives a warning.
+ * DEPRECATED: Does nothing
  * 
  **/
 void
 librdf_serializer_set_warning(librdf_serializer* serializer, void *user_data,
                               void (*warning_fn)(void *user_data, const char *msg, ...))
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(serializer, librdf_serializer);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(warning_fn, warning handler);
-
-  serializer->warning_user_data=user_data;
-  serializer->warning_fn=warning_fn;
 }
 
 
@@ -525,48 +552,136 @@ librdf_serializer_set_namespace(librdf_serializer* serializer,
 int main(int argc, char *argv[]);
 
 
+struct log_data {
+  int errors;
+  int warnings;
+} LogData;
+
+
+static int
+log_handler(void *user_data, librdf_log_message *message) 
+{
+  struct log_data* ld=(struct log_data*)user_data;
+
+  switch(message->level) {
+    case LIBRDF_LOG_ERROR:
+      ld->errors++;
+      break;
+    case LIBRDF_LOG_WARN:
+      ld->warnings++;
+      break;
+    default:
+      break;
+  }
+
+  return 1;
+}
+
+
+#define EXPECTED_ERRORS 3
+#define EXPECTED_WARNINGS 0
+
+
 int
 main(int argc, char *argv[]) 
 {
-  librdf_serializer* d;
-  char *test_serializer_types[]={"ntriples", NULL};
+  char *program=argv[0];
+  char *test_serializer_types[]={"rdfxml", "ntriples", NULL};
   int i;
   char *type;
-  char *program=argv[0];
+  unsigned char *string;
+  size_t string_length;
   librdf_world *world;
+  librdf_storage *storage;
+  librdf_model* model;
+  librdf_uri* base_uri;
+  librdf_statement* statement;
+  librdf_serializer* serializer;
 
   world=librdf_new_world();
-  librdf_world_init_mutex(world);
+  librdf_world_open(world);
 
-  /* Needed for URI use when registering factories */
-  librdf_init_digest(world);
-  librdf_init_hash(world);
-  librdf_init_uri(world);
+  librdf_world_set_logger(world, &LogData, log_handler);
 
-  /* initialise serializer module */
-  librdf_init_serializer(world);
-  
   for(i=0; (type=test_serializer_types[i]); i++) {
     fprintf(stderr, "%s: Trying to create new %s serializer\n", program, type);
-    d=librdf_new_serializer(world, type, NULL, NULL);
-    if(!d) {
+    serializer=librdf_new_serializer(world, type, NULL, NULL);
+    if(!serializer) {
       fprintf(stderr, "%s: Failed to create new serializer type %s\n", program, type);
       continue;
     }
     
     fprintf(stderr, "%s: Freeing serializer\n", program);
-    librdf_free_serializer(d);
+    librdf_free_serializer(serializer);
   }
   
+
+  storage=librdf_new_storage(world, NULL, NULL, NULL);
+  model=librdf_new_model(world, storage, NULL);
+
+  /* ERROR: Subject URI is bad UTF-8 */
+  statement=librdf_new_statement_from_nodes(world,
+    librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/foo\xfc"),
+    librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/bar"),
+    librdf_new_node_from_literal(world, (const unsigned char*)"blah", NULL, 0));
+
+  librdf_model_add_statement(model, statement);
+  librdf_free_statement(statement);
+
+  /* ERROR: Predicate URI is not serializable */
+  statement=librdf_new_statement_from_nodes(world,
+    librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/foo"),
+    librdf_new_node_from_uri_string(world, (const unsigned char*)"http://bad.example.org/"),
+    librdf_new_node_from_literal(world, (const unsigned char*)"blah", NULL, 0));
+
+  librdf_model_add_statement(model, statement);
+  librdf_free_statement(statement);
+
+  /* ERROR: Object literal is bad UTF-8 */
+  statement=librdf_new_statement_from_nodes(world,
+    librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/foo"),
+    librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/abc"),
+    librdf_new_node_from_literal(world, (const unsigned char*)"\xfc", NULL, 0));
+
+  librdf_model_add_statement(model, statement);
+  librdf_free_statement(statement);
+
+  serializer=librdf_new_serializer(world, "rdfxml", NULL, NULL);
+  base_uri=librdf_new_uri(world, (const unsigned char*)"http://example.org/base#");
+
+  string=librdf_serializer_serialize_model_to_counted_string(serializer,
+                                                             base_uri, model,
+                                                             &string_length);
+#define EXPECTED_BAD_STRING_LENGTH 237
+  if(string_length != EXPECTED_BAD_STRING_LENGTH) {
+    fprintf(stderr, "%s: Serialising to RDF/XML returned string '%s' size %d, expected %d\n", program, string,
+            string_length, EXPECTED_BAD_STRING_LENGTH);
+    return 1;
+  }
+
+  if(string)
+    free(string);
+
+  librdf_free_serializer(serializer);
+
+  librdf_free_uri(base_uri);
   
-  /* finish serializer module */
-  librdf_finish_serializer(world);
+  librdf_free_model(model);
+  librdf_free_storage(storage);
 
-  librdf_finish_uri(world);
-  librdf_finish_hash(world);
-  librdf_finish_digest(world);
+  if(LogData.errors != EXPECTED_ERRORS) {
+    fprintf(stderr, "%s: Serialising to RDF/XML returned %d errors, expected %d\n", program,
+            LogData.errors, EXPECTED_ERRORS);
+    return 1;
+  }
 
-  LIBRDF_FREE(librdf_world, world);
+  if(LogData.warnings != EXPECTED_WARNINGS) {
+    fprintf(stderr, "%s: Serialising to RDF/XML returned %d warnings, expected %d\n", program,
+            LogData.warnings, EXPECTED_WARNINGS);
+    return 1;
+  }
+  
+  librdf_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
