@@ -2,26 +2,37 @@
  *
  * rdf_statement.c - RDF Statement implementation
  *
- * $Id: rdf_statement.c,v 1.66 2004/01/14 21:28:55 cmdjb Exp $
+ * $Id: rdf_statement.c,v 1.73 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -29,7 +40,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 
 
 #ifndef STANDALONE
@@ -97,23 +108,23 @@ librdf_new_statement_from_statement(librdf_statement* statement)
   if(!new_statement)
     return NULL;
 
-  if(LIBRDF_NODE_STATEMENT_SUBJECT(statement)) {
-    LIBRDF_NODE_STATEMENT_SUBJECT(new_statement)=librdf_new_node_from_node(LIBRDF_NODE_STATEMENT_SUBJECT(statement));
-    if(!LIBRDF_NODE_STATEMENT_SUBJECT(new_statement)) {
+  if(statement->subject) {
+    new_statement->subject=librdf_new_node_from_node(statement->subject);
+    if(!new_statement->subject) {
       librdf_free_statement(new_statement);
       return NULL;
     }
   }
-  if(LIBRDF_NODE_STATEMENT_PREDICATE(statement)) {
-    LIBRDF_NODE_STATEMENT_PREDICATE(new_statement)=librdf_new_node_from_node(LIBRDF_NODE_STATEMENT_PREDICATE(statement));
-    if(!LIBRDF_NODE_STATEMENT_PREDICATE(new_statement)) {
+  if(statement->predicate) {
+    new_statement->predicate=librdf_new_node_from_node(statement->predicate);
+    if(!new_statement->predicate) {
       librdf_free_statement(new_statement);
       return NULL;
     }
   }
-  if(LIBRDF_NODE_STATEMENT_OBJECT(statement)) {
-    LIBRDF_NODE_STATEMENT_OBJECT(new_statement)=librdf_new_node_from_node(LIBRDF_NODE_STATEMENT_OBJECT(statement));
-    if(!LIBRDF_NODE_STATEMENT_OBJECT(new_statement)) {
+  if(statement->object) {
+    new_statement->object=librdf_new_node_from_node(statement->object);
+    if(!new_statement->object) {
       librdf_free_statement(new_statement);
       return NULL;
     }
@@ -153,9 +164,9 @@ librdf_new_statement_from_nodes(librdf_world *world,
     return NULL;
   }
   
-  LIBRDF_NODE_STATEMENT_SUBJECT(new_statement)=subject;
-  LIBRDF_NODE_STATEMENT_PREDICATE(new_statement)=predicate;
-  LIBRDF_NODE_STATEMENT_OBJECT(new_statement)=object;
+  new_statement->subject=subject;
+  new_statement->predicate=predicate;
+  new_statement->object=object;
 
   return new_statement;
 }
@@ -195,17 +206,17 @@ librdf_statement_clear(librdf_statement *statement)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
-  if(LIBRDF_NODE_STATEMENT_SUBJECT(statement)) {
-    librdf_free_node(LIBRDF_NODE_STATEMENT_SUBJECT(statement));
-    LIBRDF_NODE_STATEMENT_SUBJECT(statement)=NULL;
+  if(statement->subject) {
+    librdf_free_node(statement->subject);
+    statement->subject=NULL;
   }
-  if(LIBRDF_NODE_STATEMENT_PREDICATE(statement)) {
-    librdf_free_node(LIBRDF_NODE_STATEMENT_PREDICATE(statement));
-    LIBRDF_NODE_STATEMENT_PREDICATE(statement)=NULL;
+  if(statement->predicate) {
+    librdf_free_node(statement->predicate);
+    statement->predicate=NULL;
   }
-  if(LIBRDF_NODE_STATEMENT_OBJECT(statement)) {
-    librdf_free_node(LIBRDF_NODE_STATEMENT_OBJECT(statement));
-    LIBRDF_NODE_STATEMENT_OBJECT(statement)=NULL;
+  if(statement->object) {
+    librdf_free_node(statement->object);
+    statement->object=NULL;
   }
 }
 
@@ -218,9 +229,23 @@ librdf_statement_clear(librdf_statement *statement)
 void
 librdf_free_statement(librdf_statement* statement)
 {
+#ifdef WITH_THREADS
+  librdf_world *world;
+#endif
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
+#ifdef WITH_THREADS
+  world = node->world;
+  pthread_mutex_lock(world->statements_mutex);
+#endif
+
   librdf_statement_clear(statement);
+
+#ifdef WITH_THREADS
+  pthread_mutex_unlock(world->statements_mutex);
+#endif
+
   LIBRDF_FREE(librdf_statement, statement);
 }
 
@@ -232,15 +257,17 @@ librdf_free_statement(librdf_statement* statement)
  * librdf_statement_get_subject - Get the statement subject
  * @statement: &librdf_statement object
  * 
+ * This method returns a SHARED pointer to the subject which must
+ * be copied by the caller if needed.
+ * 
  * Return value: a pointer to the &librdf_node of the statement subject - 
- * NOTE this is a shared copy and must be copied if used by the caller.
  **/
 librdf_node*
 librdf_statement_get_subject(librdf_statement *statement) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, NULL);
 
-  return LIBRDF_NODE_STATEMENT_SUBJECT(statement);
+  return statement->subject;
 }
 
 
@@ -257,7 +284,7 @@ librdf_statement_set_subject(librdf_statement *statement, librdf_node *node)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
-  LIBRDF_NODE_STATEMENT_SUBJECT(statement)=node;
+  statement->subject=node;
 }
 
 
@@ -265,15 +292,17 @@ librdf_statement_set_subject(librdf_statement *statement, librdf_node *node)
  * librdf_statement_get_predicate - Get the statement predicate
  * @statement: &librdf_statement object
  * 
+ * This method returns a SHARED pointer to the predicate which must
+ * be copied by the caller if needed.
+ * 
  * Return value: a pointer to the &librdf_node of the statement predicate - 
- * NOTE this is a shared copy and must be copied if used by the caller.
  **/
 librdf_node*
 librdf_statement_get_predicate(librdf_statement *statement) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, NULL);
 
-  return LIBRDF_NODE_STATEMENT_PREDICATE(statement);
+  return statement->predicate;
 }
 
 
@@ -281,7 +310,7 @@ librdf_statement_get_predicate(librdf_statement *statement)
  * librdf_statement_set_predicate - Set the statement predicate
  * @statement: &librdf_statement object
  * @node: &librdf_node of predicate
- * 
+ *
  * The predicate passed in becomes owned by
  * the statement object and must not be used by the caller after this call.
  **/
@@ -290,7 +319,7 @@ librdf_statement_set_predicate(librdf_statement *statement, librdf_node *node)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
-  LIBRDF_NODE_STATEMENT_PREDICATE(statement)=node;
+  statement->predicate=node;
 }
 
 
@@ -298,15 +327,17 @@ librdf_statement_set_predicate(librdf_statement *statement, librdf_node *node)
  * librdf_statement_get_object - Get the statement object
  * @statement: &librdf_statement object
  * 
+ * This method returns a SHARED pointer to the object which must
+ * be copied by the caller if needed.
+ * 
  * Return value: a pointer to the &librdf_node of the statement object - 
- * NOTE this is a shared copy and must be copied if used by the caller.
  **/
 librdf_node*
 librdf_statement_get_object(librdf_statement *statement) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, NULL);
 
-  return LIBRDF_NODE_STATEMENT_OBJECT(statement);
+  return statement->object;
 }
 
 
@@ -323,7 +354,7 @@ librdf_statement_set_object(librdf_statement *statement, librdf_node *node)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
-  LIBRDF_NODE_STATEMENT_OBJECT(statement)=node;
+  statement->object=node;
 }
 
 
@@ -338,9 +369,7 @@ librdf_statement_is_complete(librdf_statement *statement)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 0);
 
-  return (LIBRDF_NODE_STATEMENT_SUBJECT(statement) &&
-          LIBRDF_NODE_STATEMENT_PREDICATE(statement) &&
-          LIBRDF_NODE_STATEMENT_OBJECT(statement));
+  return (statement->subject && statement->predicate && statement->object);
 }
 
 
@@ -366,8 +395,8 @@ librdf_statement_to_string(librdf_statement *statement)
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, NULL);
 
-  if(LIBRDF_NODE_STATEMENT_SUBJECT(statement)) {
-    subject_string=librdf_node_to_counted_string(LIBRDF_NODE_STATEMENT_SUBJECT(statement), &len);
+  if(statement->subject) {
+    subject_string=librdf_node_to_counted_string(statement->subject, &len);
     if(!subject_string)
       return NULL;
     statement_string_len += len;
@@ -377,8 +406,8 @@ librdf_statement_to_string(librdf_statement *statement)
   }
 
   
-  if(LIBRDF_NODE_STATEMENT_PREDICATE(statement)) {
-    predicate_string=librdf_node_to_counted_string(LIBRDF_NODE_STATEMENT_PREDICATE(statement), &len);
+  if(statement->predicate) {
+    predicate_string=librdf_node_to_counted_string(statement->predicate, &len);
     if(!predicate_string) {
       if(subject_string != null_string)
         LIBRDF_FREE(cstring, subject_string);
@@ -391,8 +420,8 @@ librdf_statement_to_string(librdf_statement *statement)
   }
   
 
-  if(LIBRDF_NODE_STATEMENT_OBJECT(statement)) {
-    object_string=librdf_node_to_counted_string(LIBRDF_NODE_STATEMENT_OBJECT(statement), &len);
+  if(statement->object) {
+    object_string=librdf_node_to_counted_string(statement->object, &len);
     if(!object_string) {
       if(subject_string != null_string)
         LIBRDF_FREE(cstring, subject_string);
@@ -414,8 +443,8 @@ librdf_statement_to_string(librdf_statement *statement)
                             2 + /* ", " %s */
                             2 + /* ", " %s */
                             1; /* "}" */
-  if(LIBRDF_NODE_STATEMENT_OBJECT(statement) &&
-     librdf_node_get_type(LIBRDF_NODE_STATEMENT_OBJECT(statement)) == LIBRDF_NODE_TYPE_LITERAL) {
+  if(statement->object &&
+     librdf_node_get_type(statement->object) == LIBRDF_NODE_TYPE_LITERAL) {
     format=LIBRDF_STATEMENT_FORMAT_STRING_LITERAL;
     statement_string_len+=2; /* Extra "" around literal */
   } else {
@@ -482,17 +511,14 @@ librdf_statement_equals(librdf_statement* statement1,
   if(!statement1 || !statement2)
     return 0;
   
-  if(!librdf_node_equals(LIBRDF_NODE_STATEMENT_SUBJECT(statement1), 
-                         LIBRDF_NODE_STATEMENT_SUBJECT(statement2)))
-      return 0;
-
-  if(!librdf_node_equals(LIBRDF_NODE_STATEMENT_PREDICATE(statement1),
-                         LIBRDF_NODE_STATEMENT_PREDICATE(statement2)))
-      return 0;
-
-  if(!librdf_node_equals(LIBRDF_NODE_STATEMENT_OBJECT(statement1),
-                         LIBRDF_NODE_STATEMENT_OBJECT(statement2)))
-      return 0;
+  if(!librdf_node_equals(statement1->subject, statement2->subject))
+    return 0;
+  
+  if(!librdf_node_equals(statement1->predicate, statement2->predicate))
+    return 0;
+  
+  if(!librdf_node_equals(statement1->object, statement2->object))
+    return 0;
 
   return 1;
 }
@@ -517,19 +543,16 @@ librdf_statement_match(librdf_statement* statement,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 0);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(partial_statement, librdf_statement, 0);
 
-  if(LIBRDF_NODE_STATEMENT_SUBJECT(partial_statement) &&
-     !librdf_node_equals(LIBRDF_NODE_STATEMENT_SUBJECT(statement), 
-                         LIBRDF_NODE_STATEMENT_SUBJECT(partial_statement)))
+  if(partial_statement->subject &&
+     !librdf_node_equals(statement->subject, partial_statement->subject))
       return 0;
 
-  if(LIBRDF_NODE_STATEMENT_PREDICATE(partial_statement) &&
-     !librdf_node_equals(LIBRDF_NODE_STATEMENT_PREDICATE(statement),
-                         LIBRDF_NODE_STATEMENT_PREDICATE(partial_statement)))
+  if(partial_statement->predicate &&
+     !librdf_node_equals(statement->predicate, partial_statement->predicate))
       return 0;
 
-  if(LIBRDF_NODE_STATEMENT_OBJECT(partial_statement) &&
-     !librdf_node_equals(LIBRDF_NODE_STATEMENT_OBJECT(statement),
-                         LIBRDF_NODE_STATEMENT_OBJECT(partial_statement)))
+  if(partial_statement->object &&
+     !librdf_node_equals(statement->object, partial_statement->object))
       return 0;
 
   return 1;
@@ -606,7 +629,7 @@ librdf_statement_encode_parts(librdf_statement* statement,
   }
   total_length++;
 
-  if((fields & LIBRDF_STATEMENT_SUBJECT) && LIBRDF_NODE_STATEMENT_SUBJECT(statement)) {
+  if((fields & LIBRDF_STATEMENT_SUBJECT) && statement->subject) {
     /* 's' + subject */
     if(p) {
       *p++='s';
@@ -614,7 +637,7 @@ librdf_statement_encode_parts(librdf_statement* statement,
     }
     total_length++;
 
-    node_len=librdf_node_encode(LIBRDF_NODE_STATEMENT_SUBJECT(statement), p, length);
+    node_len=librdf_node_encode(statement->subject, p, length);
     if(!node_len)
       return 0;
     if(p) {
@@ -627,7 +650,7 @@ librdf_statement_encode_parts(librdf_statement* statement,
   }
   
   
-  if((fields & LIBRDF_STATEMENT_PREDICATE) && LIBRDF_NODE_STATEMENT_PREDICATE(statement)) {
+  if((fields & LIBRDF_STATEMENT_PREDICATE) && statement->predicate) {
     /* 'p' + predicate */
     if(p) {
       *p++='p';
@@ -635,7 +658,7 @@ librdf_statement_encode_parts(librdf_statement* statement,
     }
     total_length++;
 
-    node_len=librdf_node_encode(LIBRDF_NODE_STATEMENT_PREDICATE(statement), p, length);
+    node_len=librdf_node_encode(statement->predicate, p, length);
     if(!node_len)
       return 0;
     if(p) {
@@ -646,7 +669,7 @@ librdf_statement_encode_parts(librdf_statement* statement,
     total_length += node_len;
   }
   
-  if((fields & LIBRDF_STATEMENT_OBJECT) && LIBRDF_NODE_STATEMENT_OBJECT(statement)) {
+  if((fields & LIBRDF_STATEMENT_OBJECT) && statement->object) {
     /* 'o' object */
     if(p) {
       *p++='o';
@@ -654,7 +677,7 @@ librdf_statement_encode_parts(librdf_statement* statement,
     }
     total_length++;
 
-    node_len= librdf_node_encode(LIBRDF_NODE_STATEMENT_OBJECT(statement), p, length);
+    node_len= librdf_node_encode(statement->object, p, length);
     if(!node_len)
       return 0;
     if(p) {
@@ -773,15 +796,15 @@ librdf_statement_decode_parts(librdf_statement* statement,
   
     switch(type) {
     case 's': /* subject */
-      LIBRDF_NODE_STATEMENT_SUBJECT(statement)=node;
+      statement->subject=node;
       break;
       
     case 'p': /* predicate */
-      LIBRDF_NODE_STATEMENT_PREDICATE(statement)=node;
+      statement->predicate=node;
       break;
       
     case 'o': /* object */
-      LIBRDF_NODE_STATEMENT_OBJECT(statement)=node;
+      statement->object=node;
       break;
 
     case 'c': /* context */
@@ -792,7 +815,9 @@ librdf_statement_decode_parts(librdf_statement* statement,
       break;
 
     default:
-      LIBRDF_ERROR2(statement->world, "Illegal statement encoding '%c' seen\n", p[-1]);
+      librdf_log(statement->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STATEMENT, NULL,
+                 "Illegal statement encoding '%c' seen\n", p[-1]);
       return 0;
     }
   }
