@@ -2,26 +2,37 @@
  *
  * rdf_storage.c - RDF Storage Implementation
  *
- * $Id: rdf_storage.c,v 1.65 2004/01/25 14:56:27 cmdjb Exp $
+ * $Id: rdf_storage.c,v 1.78 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bristol.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -31,7 +42,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 #include <rdf_storage.h>
 #include <rdf_storage_hashes.h>
 #include <rdf_storage_list.h>
@@ -67,15 +78,15 @@ void
 librdf_init_storage(librdf_world *world)
 {
 #ifdef HAVE_MYSQL
-  librdf_init_storage_mysql();
+  librdf_init_storage_mysql(world);
 #endif
 #ifdef HAVE_TSTORE
-  librdf_init_storage_tstore();
+  librdf_init_storage_tstore(world);
 #endif
   /* Always have storage list, hashes, file implementations available */
-  librdf_init_storage_file();
-  librdf_init_storage_hashes();
-  librdf_init_storage_list();
+  librdf_init_storage_file(world);
+  librdf_init_storage_hashes(world);
+  librdf_init_storage_list(world);
 }
 
 
@@ -121,13 +132,15 @@ librdf_delete_storage_factories(void)
 
 /**
  * librdf_storage_register_factory - Register a storage factory
+ * @world: redland world object
  * @name: the storage factory name
  * @label: the storage factory label
  * @factory: pointer to function to call to register the factory
  * 
  **/
 void
-librdf_storage_register_factory(const char *name, const char *label,
+librdf_storage_register_factory(librdf_world* world,
+                                const char *name, const char *label,
 				void (*factory) (librdf_storage_factory*)) 
 {
   librdf_storage_factory *storage, *h;
@@ -141,12 +154,12 @@ librdf_storage_register_factory(const char *name, const char *label,
   storage=(librdf_storage_factory*)LIBRDF_CALLOC(librdf_storage_factory, 1,
                                                  sizeof(librdf_storage_factory));
   if(!storage)
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_STORAGE, "Out of memory");
 
   name_copy=(char*)LIBRDF_CALLOC(cstring, strlen(name)+1, 1);
   if(!name_copy) {
     LIBRDF_FREE(librdf_storage, storage);
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_STORAGE, "Out of memory");
   }
   strcpy(name_copy, name);
   storage->name=name_copy;
@@ -155,7 +168,9 @@ librdf_storage_register_factory(const char *name, const char *label,
     if(!strcmp(h->name, name_copy)) {
       LIBRDF_FREE(cstring, name_copy); 
       LIBRDF_FREE(librdf_storage, storage);
-      LIBRDF_ERROR2(NULL, "storage %s already registered\n", h->name);
+      librdf_log(world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "storage %s already registered\n", h->name);
       return;
     }
   }
@@ -163,7 +178,7 @@ librdf_storage_register_factory(const char *name, const char *label,
   label_copy=(char*)LIBRDF_CALLOC(cstring, strlen(label)+1, 1);
   if(!label_copy) {
     LIBRDF_FREE(librdf_storage, storage);
-    LIBRDF_FATAL1(world, "Out of memory");
+    LIBRDF_FATAL1(world, LIBRDF_FROM_STORAGE, "Out of memory");
   }
   strcpy(label_copy, label);
   storage->label=label_copy;
@@ -344,7 +359,9 @@ librdf_new_storage_from_storage(librdf_storage* old_storage)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(old_storage, librdf_storage, NULL);
 
   if(!old_storage->factory->clone) {
-    LIBRDF_ERROR2(old_storage->world, "clone method not implemented for storage factory %s", old_storage->factory->name);
+    librdf_log(old_storage->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "clone method not implemented for storage factory %s", old_storage->factory->name);
     return NULL;
   }
 
@@ -530,7 +547,10 @@ librdf_storage_size(librdf_storage* storage)
  * @statement: &librdf_statement statement to add
  * 
  * The passed-in statement is copied when added to the store, not
- * shared with the store.
+ * shared with the store.  
+ *
+ * If the statement already exists in the store, it is not added
+ * unless Redland contexts are being used.
  *
  * Return value: non 0 on failure
  **/
@@ -552,6 +572,9 @@ librdf_storage_add_statement(librdf_storage* storage,
  * @storage: &librdf_storage object
  * @statement_stream: &librdf_stream of statements
  * 
+ * If any of the statements already exists in the store, they are not
+ * added unless Redland contexts are being used.
+ *
  * Return value: non 0 on failure
  **/
 int
@@ -749,7 +772,9 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
           break;
 
         default: /* error */
-          LIBRDF_ERROR2(statement->world, "Unknown statement part %d\n", context->want);
+          librdf_log(statement->world,
+                     0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                     "Unknown statement part %d\n", context->want);
           node=NULL;
       }
       break;
@@ -759,7 +784,9 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
       break;
       
     default:
-      LIBRDF_ERROR2(statement->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(statement->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       node=NULL;
   }
   
@@ -796,7 +823,7 @@ librdf_storage_stream_to_node_iterator_finished(void* iterator)
  * librdf_storage_node_stream_to_node_create - Create a stream for get sources, targets or arcs methods using find_statements method
  * @storage: the storage object to use
  * @node1: the first node to encode in the key
- * @node2: the second node to encode in the key
+ * @node2: the second node to encode in the key (or NULL if not needed)
  * @want: the field required from the statement
  * 
  * Return value: a new &librdf_iterator or NULL on failure
@@ -814,7 +841,6 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(node1, librdf_node, NULL);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(node2, librdf_node, NULL);
 
   partial_statement=librdf_new_statement(storage->world);
   if(!partial_statement)
@@ -841,7 +867,9 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
       break;
     default:
       librdf_free_statement(partial_statement);
-      LIBRDF_ERROR2(storage->world, "Illegal statement part %d seen\n", want);
+      librdf_log(storage->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Illegal statement part %d seen\n", want);
       return NULL;
   }
   
@@ -1076,7 +1104,10 @@ librdf_storage_context_add_statement(librdf_storage* storage,
                                      librdf_statement* statement) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_statement, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 1);
+
+  if(!context)
+    return librdf_storage_add_statement(storage, statement);
 
   if(storage->factory->context_add_statement)
     return storage->factory->context_add_statement(storage, context, statement);
@@ -1102,7 +1133,10 @@ librdf_storage_context_add_statements(librdf_storage* storage,
   int status=0;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_statement, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(stream, librdf_stream, 1);
+
+  if(!context)
+    return librdf_storage_add_statements(storage, stream);
 
   if(storage->factory->context_add_statements)
     return storage->factory->context_add_statements(storage, context, stream);
@@ -1239,8 +1273,8 @@ librdf_storage_supports_query(librdf_storage* storage, librdf_query *query)
 }
 
 
-librdf_stream*
-librdf_storage_query(librdf_storage* storage, librdf_query *query) 
+librdf_query_results*
+librdf_storage_query_execute(librdf_storage* storage, librdf_query *query) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, librdf_query, NULL);
@@ -1284,10 +1318,16 @@ librdf_storage_find_statements_in_context(librdf_storage* storage, librdf_statem
   if(storage->factory->find_statements_in_context)
     return storage->factory->find_statements_in_context(storage, statement, context_node);
 
-  stream=librdf_storage_context_as_stream(storage, context_node);
-  if(!stream)
+  statement=librdf_new_statement_from_statement(statement);
+  if(!statement)
     return NULL;
 
+  stream=librdf_storage_context_as_stream(storage, context_node);
+  if(!stream) {
+    librdf_free_statement(statement);
+    return NULL;
+  }
+
   librdf_stream_add_map(stream, 
                         &librdf_stream_statement_find_map,
                         (librdf_stream_map_free_context_handler)&librdf_free_statement, (void*)statement);
@@ -1323,7 +1363,7 @@ librdf_storage_get_contexts(librdf_storage* storage)
  * @storage: &librdf_storage object
  * @feature: &librdf_uri feature property
  * 
- * Return value: &librdf_node feature value or NULL if no such feature
+ * Return value: new &librdf_node feature value or NULL if no such feature
  * exists or the value is empty.
  **/
 librdf_node*
