@@ -2,26 +2,37 @@
  *
  * rdf_storage_hashes.c - RDF Storage as Hashes Implementation
  *
- * $Id: rdf_storage_hashes.c,v 1.66 2004/01/25 18:04:35 cmdjb Exp $
+ * $Id: rdf_storage_hashes.c,v 1.75 2004/11/01 20:43:35 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bristol.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <sys/types.h>
 
@@ -32,7 +43,7 @@
 #endif
 
 
-#include <librdf.h>
+#include <redland.h>
 #include <rdf_storage.h>
 #include <rdf_storage_hashes.h>
 
@@ -635,6 +646,10 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
 static int
 librdf_storage_hashes_add_statement(librdf_storage* storage, librdf_statement* statement)
 {
+  /* Do not add duplicate statements */
+  if(librdf_storage_hashes_contains_statement(storage, statement))
+    return 0;
+
   return librdf_storage_hashes_add_remove_statement(storage, statement, NULL, 1);
 }
 
@@ -877,11 +892,13 @@ librdf_storage_hashes_serialise_get_statement(void* context, int flags)
       case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
         return librdf_iterator_get_context(scontext->iterator);
       default:
-        LIBRDF_ERROR2(scontext->iterator->world, "Unimplemented flags %d seen\n", flags);
+        librdf_log(scontext->iterator->world,
+                   0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                   "Unimplemented flags %d seen\n", flags);
         return NULL;
     }
   }
-  
+    
   switch(flags) {
     case LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT:
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
@@ -927,7 +944,9 @@ librdf_storage_hashes_serialise_get_statement(void* context, int flags)
         return scontext->context_node;
         
     default:
-      LIBRDF_ERROR2(scontext->iterator->world, "Unimplemented flags %d seen\n", flags);
+      librdf_log(scontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unimplemented flags %d seen\n", flags);
       return NULL;
   }
 }
@@ -1074,7 +1093,9 @@ librdf_storage_hashes_node_iterator_get_method(void* iterator, int flags)
   
   
   if(flags != LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT) {
-    LIBRDF_ERROR2(context->iterator->world, "Unimplemented iterator method %d\n", flags);
+    librdf_log(context->iterator->world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "Unimplemented iterator method %d\n", flags);
     return NULL;
   }
 
@@ -1104,7 +1125,9 @@ librdf_storage_hashes_node_iterator_get_method(void* iterator, int flags)
       break;
       
     default: /* error */
-      LIBRDF_ERROR2(context->iterator->world, "Illegal statement part %d seen\n", context->want);
+      librdf_log(context->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Illegal statement part %d seen\n", context->want);
       return NULL;
   }
 
@@ -1143,7 +1166,9 @@ librdf_storage_hashes_node_iterator_get_method(void* iterator, int flags)
       break;
       
     default: /* error */
-      LIBRDF_ERROR2(context->iterator->world, "Illegal statement part %d seen\n", context->want);
+      librdf_log(context->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Illegal statement part %d seen\n", context->want);
       return NULL;
   }
   
@@ -1255,7 +1280,9 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
       
     default: /* error */
       LIBRDF_FREE(librdf_storage_hashes_node_iterator_context, icontext);
-      LIBRDF_ERROR2(storage->world, "Illegal statement part %d seen\n", want);
+      librdf_log(storage->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Illegal statement part %d seen\n", want);
       return NULL;
   }
 
@@ -1360,8 +1387,11 @@ librdf_storage_hashes_context_add_statement(librdf_storage* storage,
   int size;
   int status;
   
-  if(context->contexts_index <0)
+  if(context->contexts_index <0) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
     return 1;
+  }
   
   if(librdf_storage_hashes_add_remove_statement(storage, 
                                                 statement, context_node, 1))
@@ -1402,6 +1432,11 @@ librdf_storage_hashes_context_remove_statement(librdf_storage* storage,
   int size;
   int status;
   
+  if(context_node && context->contexts_index <0) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
+  }
+  
   if(librdf_storage_hashes_add_remove_statement(storage, 
                                                 statement, context_node, 0))
     return 1;
@@ -1452,8 +1487,11 @@ librdf_storage_hashes_context_serialise(librdf_storage* storage,
   librdf_stream* stream;
   size_t size;
 
-  if(context->contexts_index <0)
+  if(context->contexts_index <0) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
     return NULL;
+  }
   
   scontext=(librdf_storage_hashes_context_serialise_stream_context*)LIBRDF_CALLOC(librdf_storage_hashes_context_serialise_stream_context, 1, sizeof(librdf_storage_hashes_context_serialise_stream_context));
   if(!scontext)
@@ -1570,7 +1608,9 @@ librdf_storage_hashes_context_serialise_get_statement(void* context, int flags)
         return scontext->context_node;
 
     default:
-      LIBRDF_ERROR2(scontext->iterator->world, "Unimplemented flags %d seen\n", flags);
+      librdf_log(scontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unimplemented flags %d seen\n", flags);
       return NULL;
   }
 
@@ -1673,7 +1713,9 @@ librdf_storage_hashes_get_contexts_get_method(void* iterator, int flags)
       break;
       
     default:
-      LIBRDF_ERROR2(icontext->storage->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(icontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       result=NULL;
       break;
   }
@@ -1715,8 +1757,11 @@ librdf_storage_hashes_get_contexts(librdf_storage* storage)
   librdf_storage_hashes_get_contexts_iterator_context* icontext;
   librdf_iterator* iterator;
 
-  if(context->index_contexts <0)
+  if(context->index_contexts <0) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
     return NULL;
+  }
   
   icontext=(librdf_storage_hashes_get_contexts_iterator_context*)LIBRDF_CALLOC(librdf_storage_hashes_get_contexts_iterator_context, 1, sizeof(librdf_storage_hashes_get_contexts_iterator_context));
   if(!icontext)
@@ -1758,7 +1803,7 @@ librdf_storage_hashes_get_contexts(librdf_storage* storage)
  * @storage: &librdf_storage object
  * @feature: &librdf_uri feature property
  * 
- * Return value: &librdf_node feature value or NULL if no such feature
+ * Return value: new &librdf_node feature value or NULL if no such feature
  * exists or the value is empty.
  **/
 static librdf_node*
@@ -1820,8 +1865,8 @@ librdf_storage_hashes_register_factory(librdf_storage_factory *factory)
 
 
 void
-librdf_init_storage_hashes(void)
+librdf_init_storage_hashes(librdf_world *world)
 {
-  librdf_storage_register_factory("hashes", "Indexed hashes",
+  librdf_storage_register_factory(world, "hashes", "Indexed hashes",
                                   &librdf_storage_hashes_register_factory);
 }
