@@ -2,26 +2,37 @@
  *
  * rdf_storage_tstore.c - RDF Storage using 3store
  *
- * $Id: rdf_storage_tstore.c,v 1.7 2004/01/30 13:56:21 cmdjb Exp $
+ * $Id: rdf_storage_tstore.c,v 1.13 2004/11/01 20:43:35 cmdjb Exp $
  *
- * Copyright (C) 2003-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bristol.ac.uk
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -30,7 +41,7 @@
 #endif
 #include <sys/types.h>
 
-#include <librdf.h>
+#include <redland.h>
 
 #include <rdfsql/rdfsql.h>
 
@@ -168,6 +179,8 @@ librdf_storage_tstore_size(librdf_storage* storage)
 static int
 librdf_storage_tstore_add_statement(librdf_storage* storage, librdf_statement* statement)
 {
+  /* FIXME - cannot check for adding duplicate statements */
+
   return librdf_storage_tstore_context_add_statement(storage, NULL, statement);
 }
 
@@ -359,7 +372,9 @@ librdf_storage_tstore_serialise_get_statement(void* context, int flags)
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
       return NULL;
     default:
-      LIBRDF_ERROR2(scontext->storage->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(scontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       return NULL;
   }
 }
@@ -506,7 +521,9 @@ librdf_storage_tstore_find_get_statement(void* context, int flags)
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
       return NULL;
     default:
-      LIBRDF_ERROR2(scontext->storage->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(scontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       return NULL;
   }
 }
@@ -699,8 +716,8 @@ librdf_storage_tstore_register_factory(librdf_storage_factory *factory)
 
 
 void
-librdf_init_storage_tstore(void)
+librdf_init_storage_tstore(librdf_world *world)
 {
-  librdf_storage_register_factory("tstore", "AKT triplestore",
+  librdf_storage_register_factory(world, "tstore", "AKT triplestore",
                                   &librdf_storage_tstore_register_factory);
 }
