@@ -2,26 +2,37 @@
  *
  * rdf_uri.c - RDF URI Implementation
  *
- * $Id: rdf_uri.c,v 1.62 2004/01/14 21:29:06 cmdjb Exp $
+ * $Id: rdf_uri.c,v 1.67 2004/11/01 20:43:35 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -33,12 +44,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
-
-
-/* for raptor_uri_resolve_uri_reference */
-#define RAPTOR_IN_REDLAND
-#include <raptor.h>
+#include <redland.h>
 
 
 #ifndef STANDALONE
@@ -58,10 +64,10 @@ librdf_init_uri(librdf_world *world)
   if(!world->uris_hash) {
     world->uris_hash=librdf_new_hash(world, NULL);
     if(!world->uris_hash)
-      LIBRDF_FATAL1(world, "Failed to create URI hash from factory");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_URI, "Failed to create URI hash from factory");
     
     if(librdf_hash_open(world->uris_hash, NULL, 0, 1, 1, NULL))
-      LIBRDF_FATAL1(world, "Failed to open URI hash");
+      LIBRDF_FATAL1(world, LIBRDF_FROM_URI, "Failed to open URI hash");
 
     /* remember to free it later */
     world->uris_hash_allocated_here=1;
@@ -357,7 +363,7 @@ librdf_new_uri_from_filename(librdf_world* world, const char *filename) {
     return NULL;
   
   new_uri=librdf_new_uri(world, uri_string);
-  LIBRDF_FREE(cstring, uri_string);
+  raptor_free_memory(uri_string);
   return new_uri;
 }
 
@@ -404,7 +410,7 @@ librdf_free_uri (librdf_uri* uri)
   key.data=uri->string;
   key.size=uri->string_length;
   if(librdf_hash_delete_all(uri->world->uris_hash, &key) )
-    LIBRDF_FATAL1(world, "Hash deletion failed");
+    LIBRDF_FATAL1(uri->world, LIBRDF_FROM_URI, "Hash deletion failed");
 
 
   if(uri->string)
