@@ -2,23 +2,29 @@
  *
  * ntriples_parse.c - Raptor N-Triples Parser implementation
  *
- * $Id: ntriples_parse.c,v 1.74 2004/01/20 12:24:44 cmdjb Exp $
+ * $Id: ntriples_parse.c,v 1.84 2004/09/20 11:54:18 cmdjb Exp $
  *
  * N-Triples
  * http://www.w3.org/TR/rdf-testcases/#ntriples
  *
- * Copyright (C) 2001-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2001-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -28,7 +34,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -85,6 +91,7 @@ typedef struct raptor_ntriples_parser_context_s raptor_ntriples_parser_context;
 
 static int
 raptor_ntriples_parse_init(raptor_parser* rdf_parser, const char *name) {
+  /*raptor_ntriples_parser_context *ntriples_parser=(raptor_ntriples_parser_context*)rdf_parser->context; */
   return 0;
 }
 
@@ -130,6 +137,7 @@ raptor_ntriples_generate_statement(raptor_parser *parser,
                                    const unsigned char *object_literal_language,
                                    const unsigned char *object_literal_datatype)
 {
+  /* raptor_ntriples_parser_context *ntriples_parser=(raptor_ntriples_parser_context*)parser->context; */
   raptor_statement *statement=&parser->statement;
   raptor_uri *subject_uri=NULL;
   int predicate_ordinal=0;
@@ -147,8 +155,10 @@ raptor_ntriples_generate_statement(raptor_parser *parser,
     statement->subject_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
   }
 
-  if(object_literal_datatype)
-    datatype_uri=raptor_new_uri_relative_to_base(parser->base_uri, object_literal_datatype);
+ if(object_literal_datatype) {
+   datatype_uri=raptor_new_uri_relative_to_base(parser->base_uri, object_literal_datatype);
+   object_literal_language=NULL;
+ }
 
   /* Predicates in N-Triples are URIs or ordinals */
   if(!strncmp((const char*)predicate, "http://www.w3.org/1999/02/22-rdf-syntax-ns#_", 44)) {
@@ -157,10 +167,7 @@ raptor_ntriples_generate_statement(raptor_parser *parser,
       statement->predicate=(void*)&predicate_ordinal;
       statement->predicate_type=RAPTOR_IDENTIFIER_TYPE_ORDINAL;
     } else {
-      /* FIXME - could have a warning here */
-      /*
-      raptor_parser_error(parser, "Illegal rdf:_<n> property in %s.", predicate);
-      */
+      raptor_parser_error(parser, "Illegal ordinal value %d in property '%s'.", predicate_ordinal, predicate);
       predicate_ordinal=0;
     }
   }
@@ -659,7 +666,7 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
         }
 
         if(object_literal_datatype && object_literal_language) {
-          raptor_parser_error(rdf_parser, "Typed literal used with a language - ignoring the language");
+          raptor_parser_warning(rdf_parser, "Typed literal used with a language - ignoring the language");
           object_literal_language=NULL;
         }
           
@@ -950,13 +957,11 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
     if(!*ptr)
       break;
 
-    if(*ptr) {
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
-      RAPTOR_DEBUG3("found newline \\x%02x at offset %d\n", *ptr,
-                    ptr-line_start);
+    RAPTOR_DEBUG3("found newline \\x%02x at offset %d\n", *ptr,
+                  ptr-line_start);
 #endif
-      ntriples_parser->last_char=*ptr;
-    }
+    ntriples_parser->last_char=*ptr;
 
     len=ptr-line_start;
     rdf_parser->locator.column=0;
@@ -980,13 +985,6 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #endif
   }
 
-  /* exit now, no more input */
-  if(is_end) {
-    if(ptr-buffer != ntriples_parser->line_length)
-       raptor_parser_error(rdf_parser, "Junk at end of input.\"");
-    return 0;
-  }
-    
   ntriples_parser->offset=start-buffer;
 
   len=ntriples_parser->line_length - ntriples_parser->offset;
@@ -1019,6 +1017,13 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #endif    
   }
   
+  /* exit now, no more input */
+  if(is_end) {
+    if(ntriples_parser->offset != ntriples_parser->line_length)
+       raptor_parser_error(rdf_parser, "Junk at end of input.\"");
+    return 0;
+  }
+    
   return 0;
 }
 
@@ -1039,6 +1044,33 @@ raptor_ntriples_parse_start(raptor_parser *rdf_parser)
 }
 
 
+static int
+raptor_ntriples_parse_recognise_syntax(raptor_parser_factory* factory, 
+                                       const unsigned char *buffer, size_t len,
+                                       const unsigned char *identifier, 
+                                       const unsigned char *suffix, 
+                                       const char *mime_type)
+{
+  int score= 0;
+  
+  if(suffix) {
+    if(!strcmp((const char*)suffix, "nt"))
+      score=8;
+    if(!strcmp((const char*)suffix, "ttl"))
+      score=3;
+    if(!strcmp((const char*)suffix, "n3"))
+      score=1;
+  }
+  
+  if(mime_type) {
+    if(strstr((const char*)mime_type, "ntriples"))
+      score+=6;
+  }
+  
+  return score;
+}
+
+
 static void
 raptor_ntriples_parser_register_factory(raptor_parser_factory *factory) 
 {
@@ -1048,6 +1080,7 @@ raptor_ntriples_parser_register_factory(raptor_parser_factory *factory)
   factory->terminate = raptor_ntriples_parse_terminate;
   factory->start     = raptor_ntriples_parse_start;
   factory->chunk     = raptor_ntriples_parse_chunk;
+  factory->recognise_syntax = raptor_ntriples_parse_recognise_syntax;
 }
 
 
