@@ -2,20 +2,26 @@
  *
  * raptor.h - Redland Parser Toolkit for RDF (Raptor) interfaces and definition
  *
- * $Id: raptor.h,v 1.100.2.1 2004/02/23 21:42:25 cmdjb Exp $
+ * $Id: raptor.h,v 1.138 2004/10/28 21:39:26 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -31,11 +37,18 @@ extern "C" {
 
 #include <stdio.h>
 
+/* Required for va_list in raptor_vsnprintf */
+#include <stdarg.h>
+
 #ifdef WIN32
-#  ifdef RAPTOR_INTERNAL
-#    define RAPTOR_API _declspec(dllexport)
+#  ifdef RAPTOR_STATIC
+#    define RAPTOR_API
 #  else
-#    define RAPTOR_API _declspec(dllimport)
+#    ifdef RAPTOR_INTERNAL
+#      define RAPTOR_API _declspec(dllexport)
+#    else
+#      define RAPTOR_API _declspec(dllimport)
+#    endif
 #  endif
 #else
 #  define RAPTOR_API
@@ -59,19 +72,32 @@ typedef void* raptor_uri;
 
 
 /* Public statics */
-extern const char * const raptor_short_copyright_string;
-extern const char * const raptor_copyright_string;
-extern const char * const raptor_version_string;
-extern const unsigned int raptor_version_major;
-extern const unsigned int raptor_version_minor;
-extern const unsigned int raptor_version_release;
-extern const unsigned int raptor_version_decimal;
+RAPTOR_API extern const char * const raptor_short_copyright_string;
+RAPTOR_API extern const char * const raptor_copyright_string;
+RAPTOR_API extern const char * const raptor_version_string;
+RAPTOR_API extern const unsigned int raptor_version_major;
+RAPTOR_API extern const unsigned int raptor_version_minor;
+RAPTOR_API extern const unsigned int raptor_version_release;
+RAPTOR_API extern const unsigned int raptor_version_decimal;
+
+RAPTOR_API extern const char * const raptor_xml_namespace_uri;
+RAPTOR_API extern const char * const raptor_rdf_namespace_uri;
+RAPTOR_API extern const char * const raptor_rdf_schema_namespace_uri;
+RAPTOR_API extern const char * const raptor_xmlschema_datatypes_namespace_uri;
+RAPTOR_API extern const char * const raptor_owl_namespace_uri;
+
+RAPTOR_API extern const unsigned int raptor_rdf_namespace_uri_len;
+
+RAPTOR_API extern const char * const raptor_xml_literal_datatype_uri_string;
+RAPTOR_API extern const unsigned int raptor_xml_literal_datatype_uri_string_len;
 
 
 /* Public structure */
 typedef struct raptor_parser_s raptor_parser;
+typedef struct raptor_serializer_s raptor_serializer;
 
 typedef struct raptor_www_s raptor_www;
+typedef struct raptor_iostream_s raptor_iostream;
 
 typedef struct raptor_sax2_element_s raptor_sax2_element;
 typedef struct raptor_xml_writer_s raptor_xml_writer;
@@ -115,7 +141,10 @@ typedef enum {
   RAPTOR_FEATURE_ALLOW_BAGID,
   RAPTOR_FEATURE_ALLOW_RDF_TYPE_RDF_LIST,
   RAPTOR_FEATURE_NORMALIZE_LANGUAGE,
-  RAPTOR_FEATURE_NON_NFC_FATAL
+  RAPTOR_FEATURE_NON_NFC_FATAL,
+  RAPTOR_FEATURE_WARN_OTHER_PARSETYPES,
+  RAPTOR_FEATURE_CHECK_RDF_ID,
+  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_CHECK_RDF_ID
 } raptor_feature;
 
 
@@ -196,10 +225,11 @@ RAPTOR_API void raptor_finish(void);
 RAPTOR_API int raptor_parsers_enumerate(const unsigned int counter, const char **name, const char **label);
 RAPTOR_API int raptor_syntaxes_enumerate(const unsigned int counter, const char **name, const char **label, const char **mime_type, const unsigned char **uri_string);
 RAPTOR_API int raptor_syntax_name_check(const char *name);
-
+RAPTOR_API const char* raptor_guess_parser_name(raptor_uri *uri, const char *mime_type, const unsigned char *buffer, size_t len, const unsigned char *identifier);
 
 /* Create */
 RAPTOR_API raptor_parser* raptor_new_parser(const char *name);
+RAPTOR_API raptor_parser* raptor_new_parser_for_content(raptor_uri *uri, const char *mime_type, const unsigned char *buffer, size_t len, const unsigned char *identifier);
 
 RAPTOR_API int raptor_start_parse(raptor_parser *rdf_parser, raptor_uri *uri);
 
@@ -213,9 +243,9 @@ RAPTOR_API void raptor_set_warning_handler(raptor_parser* parser, void *user_dat
 RAPTOR_API void raptor_set_statement_handler(raptor_parser* parser, void *user_data, raptor_statement_handler handler);
 RAPTOR_API void raptor_set_generate_id_handler(raptor_parser* parser, void *user_data, raptor_generate_id_handler handler);
 
-RAPTOR_API void raptor_print_statement(const raptor_statement * const statement, FILE *stream);
+RAPTOR_API void raptor_print_statement(const raptor_statement * statement, FILE *stream);
 RAPTOR_API void raptor_print_statement_as_ntriples(const raptor_statement * statement, FILE *stream);
-RAPTOR_API void raptor_print_statement_detailed(const raptor_statement * statement, int detailed, FILE *stream);
+RAPTOR_API RAPTOR_DEPRECATED void raptor_print_statement_detailed(const raptor_statement * statement, int detailed, FILE *stream);
 RAPTOR_API unsigned char* raptor_statement_part_as_counted_string(const void *term, raptor_identifier_type type, raptor_uri* literal_datatype, const unsigned char *literal_language, size_t* len_p);
 RAPTOR_API unsigned char* raptor_statement_part_as_string(const void *term, raptor_identifier_type type, raptor_uri* literal_datatype, const unsigned char *literal_language);  
 
@@ -234,12 +264,47 @@ RAPTOR_API void raptor_parse_abort(raptor_parser* rdf_parser);
 /* Utility functions */
 RAPTOR_API void raptor_print_locator(FILE *stream, raptor_locator* locator);
 RAPTOR_API int raptor_format_locator(char *buffer, size_t length, raptor_locator* locator);
- 
+RAPTOR_API int raptor_locator_line(raptor_locator *locator);
+RAPTOR_API int raptor_locator_column(raptor_locator *locator);
+RAPTOR_API int raptor_locator_byte(raptor_locator *locator);
+RAPTOR_API const char * raptor_locator_file(raptor_locator *locator);
+RAPTOR_API const char * raptor_locator_uri(raptor_locator *locator);
+
+
 RAPTOR_API const char* raptor_get_name(raptor_parser *rdf_parser);
 RAPTOR_API const char* raptor_get_label(raptor_parser *rdf_parser);
-RAPTOR_API void raptor_set_feature(raptor_parser *parser, raptor_feature feature, int value);
+RAPTOR_API const char* raptor_get_mime_type(raptor_parser *rdf_parser);
+
+RAPTOR_API int raptor_features_enumerate(const raptor_feature feature, const char **name, raptor_uri **uri, const char **label);
+RAPTOR_API int raptor_set_feature(raptor_parser *parser, raptor_feature feature, int value);
+RAPTOR_API int raptor_get_feature(raptor_parser *parser, raptor_feature feature);
+RAPTOR_API raptor_feature raptor_feature_from_uri(raptor_uri *uri);
 RAPTOR_API void raptor_set_parser_strict(raptor_parser* rdf_parser, int is_strict);
 
+/* Get serializer names */
+RAPTOR_API int raptor_serializers_enumerate(const unsigned int counter, const char **name, const char **label, const char **mime_type, const unsigned char **uri_string);
+RAPTOR_API int raptor_serializer_syntax_name_check(const char *name);
+
+/* Serializing */
+RAPTOR_API raptor_serializer* raptor_new_serializer(const char *name);
+RAPTOR_API void raptor_free_serializer(raptor_serializer* rdf_serializer);
+
+RAPTOR_API int raptor_serialize_start(raptor_serializer *rdf_serializer, raptor_uri *uri, raptor_iostream *iostream);
+RAPTOR_API int raptor_serialize_start_to_filename(raptor_serializer *rdf_serializer, const char *filename);
+RAPTOR_API int raptor_serialize_start_to_string(raptor_serializer *rdf_serializer, raptor_uri *uri, void **string_p, size_t *length_p);
+RAPTOR_API int raptor_serialize_start_to_file_handle(raptor_serializer *rdf_serializer, raptor_uri *uri, FILE *fh);
+RAPTOR_API int raptor_serialize_statement(raptor_serializer* rdf_serializer, const raptor_statement *statement);
+RAPTOR_API int raptor_serialize_end(raptor_serializer *rdf_serializer);
+RAPTOR_API raptor_iostream* raptor_serializer_get_iostream(raptor_serializer *serializer);
+RAPTOR_API void raptor_serializer_set_error_handler(raptor_serializer* serializer, void *user_data, raptor_message_handler handler);
+RAPTOR_API void raptor_serializer_set_warning_handler(raptor_serializer* serializer, void *user_data, raptor_message_handler handler);
+RAPTOR_API raptor_locator* raptor_serializer_get_locator(raptor_serializer *rdf_serializer);
+
+/* memory functions */
+RAPTOR_API void raptor_free_memory(void *ptr);
+RAPTOR_API void* raptor_alloc_memory(size_t size);
+RAPTOR_API void* raptor_calloc_memory(size_t nmemb, size_t size);
+
 /* URI functions */
 RAPTOR_API raptor_uri* raptor_new_uri(const unsigned char *uri_string);
 RAPTOR_API raptor_uri* raptor_new_uri_from_uri_local_name(raptor_uri *uri, const unsigned char *local_name);
@@ -266,13 +331,17 @@ RAPTOR_API void raptor_free_identifier(raptor_identifier *identifier);
 RAPTOR_API int raptor_print_ntriples_string(FILE *stream, const unsigned char *string, const char delim);
 RAPTOR_API unsigned char* raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser, unsigned char *src, int len, size_t *dest_lenp);
 RAPTOR_API const char* raptor_ntriples_term_as_string (raptor_ntriples_term_type term);
-RAPTOR_API size_t raptor_xml_escape_string(const unsigned char *string, size_t len, unsigned char *buffer, size_t length, char quote, raptor_simple_message_handler error_handler, void *error_data);
+RAPTOR_API int raptor_xml_escape_string(const unsigned char *string, size_t len, unsigned char *buffer, size_t length, char quote, raptor_simple_message_handler error_handler, void *error_data);
+
+RAPTOR_API char* raptor_vsnprintf(const char *message, va_list arguments);
 
 /* raptor_xml_writer.c */
 /* NOT PUBLIC YET - SEE raptor_internal.h */
 
-/* raptor_uri.c */
+/* raptor_rfc2396.c */
 RAPTOR_API void raptor_uri_resolve_uri_reference (const unsigned char *base_uri, const unsigned char *reference_uri, char unsigned *buffer, size_t length);
+
+/* raptor_uri.c */
 RAPTOR_API unsigned char *raptor_uri_filename_to_uri_string(const char *filename);
 RAPTOR_API char *raptor_uri_uri_string_to_filename(const unsigned char *uri_string);
 RAPTOR_API char *raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string, unsigned char **fragment_p);
@@ -285,6 +354,9 @@ RAPTOR_API void raptor_uri_get_handler(raptor_uri_handler **handler, void **cont
 #define RAPTOR_RDF_MS_URI "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 #define RAPTOR_RDF_SCHEMA_URI "http://www.w3.org/2000/01/rdf-schema#"
 
+#define RAPTOR_XMLSCHEMA_DATATYPES_URI "http://www.w3.org/2001/XMLSchema#"
+#define RAPTOR_OWL_URI "http://www.w3.org/2002/07/owl#"
+
 
 /* raptor_www */
 RAPTOR_API void raptor_www_init(void);
@@ -297,11 +369,12 @@ RAPTOR_API raptor_www *raptor_www_new_with_connection(void* connection);
 RAPTOR_API void raptor_www_free(raptor_www *www);
 RAPTOR_API void raptor_www_set_user_agent(raptor_www *www, const char *user_agent);
 RAPTOR_API void raptor_www_set_proxy(raptor_www *www, const char *proxy);
-void
-RAPTOR_API raptor_www_set_write_bytes_handler(raptor_www *www, raptor_www_write_bytes_handler handler, void *user_data);
+RAPTOR_API void raptor_www_set_http_accept(raptor_www *www, const char *value);
+RAPTOR_API void raptor_www_set_write_bytes_handler(raptor_www *www, raptor_www_write_bytes_handler handler, void *user_data);
 RAPTOR_API void raptor_www_set_content_type_handler(raptor_www *www, raptor_www_content_type_handler handler, void *user_data);
 RAPTOR_API void raptor_www_set_error_handler(raptor_www *www, raptor_message_handler error_handler, void *error_data);
 RAPTOR_API int raptor_www_fetch(raptor_www *www, raptor_uri *uri);
+RAPTOR_API int raptor_www_fetch_to_string(raptor_www *www, raptor_uri *uri, void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
 RAPTOR_API void* raptor_www_get_connection(raptor_www *www);
 RAPTOR_API void raptor_www_abort(raptor_www *www, const char *reason);
 
@@ -336,6 +409,86 @@ RAPTOR_API unsigned char *raptor_namespaces_format(const raptor_namespace *ns, s
 
 typedef struct raptor_stringbuffer_s raptor_stringbuffer;
 
+/* Sequence class */
+typedef struct raptor_sequence_s raptor_sequence;
+typedef void* (raptor_sequence_free_handler(void*));
+typedef void (raptor_sequence_print_handler(void *object, FILE *fh));
+
+/* Create */
+RAPTOR_API raptor_sequence* raptor_new_sequence(raptor_sequence_free_handler* free_handler, raptor_sequence_print_handler* print_handler);
+/* Destroy */
+RAPTOR_API void raptor_free_sequence(raptor_sequence* seq);
+/* Methods */
+RAPTOR_API int raptor_sequence_size(raptor_sequence* seq);
+RAPTOR_API int raptor_sequence_set_at(raptor_sequence* seq, int idx, void *data);
+RAPTOR_API int raptor_sequence_push(raptor_sequence* seq, void *data);
+RAPTOR_API int raptor_sequence_shift(raptor_sequence* seq, void *data);
+RAPTOR_API void* raptor_sequence_get_at(raptor_sequence* seq, int idx);
+RAPTOR_API void* raptor_sequence_pop(raptor_sequence* seq);
+RAPTOR_API void* raptor_sequence_unshift(raptor_sequence* seq);
+
+RAPTOR_API int raptor_compare_strings(const void *a, const void *b);
+
+RAPTOR_API void raptor_sequence_sort(raptor_sequence* seq, int(*compare)(const void *, const void *));
+
+/* helper for printing sequences of strings */ 
+RAPTOR_API void raptor_sequence_print_string(char *data, FILE *fh);
+RAPTOR_API void raptor_sequence_print_uri(char *data, FILE *fh);
+RAPTOR_API void raptor_sequence_set_print_handler(raptor_sequence *seq, raptor_sequence_print_handler *print_handler);
+RAPTOR_API void raptor_sequence_print(raptor_sequence* seq, FILE* fh);
+
+/* raptor_utf8.c */
+RAPTOR_API int raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output);
+RAPTOR_API int raptor_utf8_to_unicode_char(unsigned long *output, const unsigned char *input, int length);
+
+
+/* raptor_stringbuffer */
+RAPTOR_API raptor_stringbuffer* raptor_new_stringbuffer(void);
+RAPTOR_API void raptor_free_stringbuffer(raptor_stringbuffer *stringbuffer);
+RAPTOR_API int raptor_stringbuffer_append_counted_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, size_t length, int do_copy);
+RAPTOR_API int raptor_stringbuffer_append_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, int do_copy);
+RAPTOR_API int raptor_stringbuffer_append_decimal(raptor_stringbuffer* stringbuffer, int integer);
+RAPTOR_API int raptor_stringbuffer_append_stringbuffer(raptor_stringbuffer* stringbuffer, raptor_stringbuffer* append);
+RAPTOR_API int raptor_stringbuffer_prepend_counted_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, size_t length, int do_copy);
+RAPTOR_API int raptor_stringbuffer_prepend_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, int do_copy);
+RAPTOR_API unsigned char * raptor_stringbuffer_as_string(raptor_stringbuffer* stringbuffer);
+RAPTOR_API size_t raptor_stringbuffer_length(raptor_stringbuffer* stringbuffer);
+RAPTOR_API int raptor_stringbuffer_copy_to_string(raptor_stringbuffer* stringbuffer, unsigned char *string, size_t length);
+
+typedef int (*raptor_iostream_init_func) (void *context);
+typedef void (*raptor_iostream_finish_func) (void *context);
+typedef int (*raptor_iostream_write_byte_func) (void *context, const int byte);
+typedef int (*raptor_iostream_write_bytes_func) (void *context, const void *ptr, size_t size, size_t nmemb);
+typedef void (*raptor_iostream_write_end_func) (void *context);
+
+typedef struct {
+  /* optional, called at most once */
+  raptor_iostream_init_func         init;
+  /* optional, called at most once */
+  raptor_iostream_finish_func       finish;
+  /* methods */
+  raptor_iostream_write_byte_func   write_byte;
+  raptor_iostream_write_bytes_func  write_bytes;
+   /* optional, called at most once */
+  raptor_iostream_write_end_func    write_end;
+} raptor_iostream_handler;
+
+
+RAPTOR_API raptor_iostream* raptor_new_iostream_from_handler(void *context, raptor_iostream_handler *handler);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_sink(void);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_filename(const char *filename);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_file_handle(FILE *handle);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_string(void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
+RAPTOR_API void raptor_free_iostream(raptor_iostream *iostr);
+RAPTOR_API int raptor_iostream_write_bytes(raptor_iostream *iostr, const void *ptr, size_t size, size_t nmemb);
+RAPTOR_API int raptor_iostream_write_byte(raptor_iostream *iostr, const int byte);
+RAPTOR_API void raptor_iostream_write_end(raptor_iostream *iostr);
+RAPTOR_API int raptor_iostream_write_string(raptor_iostream *iostr, const void *string);
+RAPTOR_API int raptor_iostream_write_counted_string(raptor_iostream *iostr, const void *string, size_t len);
+RAPTOR_API size_t raptor_iostream_get_bytes_written_count(raptor_iostream *iostr);
+RAPTOR_API int raptor_iostream_write_decimal(raptor_iostream* iostr, int integer);
+RAPTOR_API int raptor_iostream_format_hexadecimal(raptor_iostream* iostr, unsigned int integer, int width);
+
 #ifdef __cplusplus
 }
 #endif
