@@ -2,20 +2,26 @@
  *
  * raptor_internal.h - Redland Parser Toolkit for RDF (Raptor) internals
  *
- * $Id: raptor_internal.h,v 1.91 2004/01/20 12:24:17 cmdjb Exp $
+ * $Id: raptor_internal.h,v 1.116 2004/10/20 15:22:16 cmdjb Exp $
  *
- * Copyright (C) 2002-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -45,7 +51,8 @@ extern "C" {
 #endif
 
 #define RAPTOR_MALLOC(type, size) malloc(size)
-#define RAPTOR_CALLOC(type, size, count) calloc(size, count)
+#define RAPTOR_CALLOC(type, nmemb, size) calloc(nmemb, size)
+#define RAPTOR_REALLOC(type, ptr, size) realloc(ptr, size)
 #define RAPTOR_FREE(type, ptr)   free((void*)ptr)
 
 #ifdef RAPTOR_DEBUG
@@ -67,6 +74,8 @@ void raptor_system_free(void *ptr);
 #define SYSTEM_FREE(ptr)   free(ptr)
 #endif
 
+#define RAPTOR_ASSERT_DIE abort();
+
 #else
 /* DEBUGGING TURNED OFF */
 
@@ -81,12 +90,57 @@ void raptor_system_free(void *ptr);
 #define SYSTEM_MALLOC(size)   malloc(size)
 #define SYSTEM_FREE(ptr)   free(ptr)
 
+#define RAPTOR_ASSERT_DIE
+
+#endif
+
+
+#ifdef RAPTOR_DISABLE_ASSERT_MESSAGES
+#define RAPTOR_ASSERT_REPORT(line)
+#else
+#define RAPTOR_ASSERT_REPORT(msg) fprintf(stderr, "%s:%d: (%s) assertion failed: " msg "\n", __FILE__, __LINE__, __func__);
+#endif
+
+
+#ifdef RAPTOR_DISABLE_ASSERT
+
+#define RAPTOR_ASSERT_RETURN(condition, msg, ret) 
+#define RAPTOR_ASSERT_OBJECT_POINTER_RETURN(pointer, type)
+#define RAPTOR_ASSERT_OBJECT_POINTER_RETURN_VALUE(pointer, type, ret)
+
+#else
+
+#define RAPTOR_ASSERT_RETURN(condition, msg, ret) do { \
+  if(condition) { \
+    RAPTOR_ASSERT_REPORT(msg) \
+    RAPTOR_ASSERT_DIE \
+    return(ret); \
+  } \
+} while(0)
+
+#define RAPTOR_ASSERT_OBJECT_POINTER_RETURN(pointer, type) do { \
+  if(!pointer) { \
+    RAPTOR_ASSERT_REPORT("object pointer of type " #type " is NULL.") \
+    RAPTOR_ASSERT_DIE \
+    return; \
+  } \
+} while(0)
+
+#define RAPTOR_ASSERT_OBJECT_POINTER_RETURN_VALUE(pointer, type, ret) do { \
+  if(!pointer) { \
+    RAPTOR_ASSERT_REPORT("object pointer of type " #type " is NULL.") \
+    RAPTOR_ASSERT_DIE \
+    return(ret); \
+  } \
+} while(0)
+
 #endif
 
 
 /* Fatal errors - always happen */
 #define RAPTOR_FATAL1(msg) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__); abort();} while(0)
 #define RAPTOR_FATAL2(msg,arg) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__, arg); abort();} while(0)
+#define RAPTOR_FATAL3(msg,arg1,arg2) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__, arg1, arg2); abort();} while(0)
 
 
 
@@ -140,6 +194,7 @@ extern void raptor_libxml_free_entities(raptor_parser *rdf_parser);
 #endif
 extern void raptor_libxml_validation_error(void *context, const char *msg, ...);
 extern void raptor_libxml_validation_warning(void *context, const char *msg, ...);
+void raptor_libxml_free(xmlParserCtxtPtr xc);
 
 /* raptor_parse.c - exported to libxml part */
 extern void raptor_libxml_update_document_locator (raptor_parser *rdf_parser);
@@ -156,9 +211,24 @@ extern void raptor_set_libxml_entities(raptor_parser *rdf_parser, raptor_xml_ent
 #endif
 
 
+/* expat-only prototypes */
+
+#ifdef RAPTOR_XML_EXPAT
+/* raptor_expat.c exports */
+extern XML_Parser raptor_expat_init(raptor_parser *rdf_parser);
+
+/* raptor_parse.c */
+void raptor_xml_unparsed_entity_decl_handler(void *user_data, const XML_Char *entityName, const XML_Char *base, const XML_Char *systemId, const XML_Char *publicId, const XML_Char *notationName);
+int raptor_xml_external_entity_ref_handler(void *user_data, const XML_Char *context, const XML_Char *base, const XML_Char *systemId, const XML_Char *publicId);
+
+/* end of expat-only */
+#endif
+
 
 typedef struct raptor_parser_factory_s raptor_parser_factory;
-typedef struct raptor_set_s raptor_set;
+typedef struct raptor_serializer_factory_s raptor_serializer_factory;
+typedef struct raptor_id_set_s raptor_id_set;
+typedef struct raptor_uri_detail_s raptor_uri_detail;
 
 
 /* Raptor Namespace Stack node */
@@ -276,6 +346,17 @@ struct raptor_parser_s {
    */
   int feature_non_nfc_fatal;
 
+  /* FEATURE:
+   * non 0 to give a warning when other rdf:parseType values that are not
+   * Literal, Resource or Collection are seen.
+   */
+  int feature_warn_other_parseTypes;
+
+  /* FEATURE:
+   * non 0 to check rdf:ID values for duplicates
+   */
+  int feature_check_rdf_id;
+
   /* stuff for our user */
   void *user_data;
 
@@ -341,15 +422,92 @@ struct raptor_parser_factory_s {
   /* finish the parser factory */
   void (*finish_factory)(raptor_parser_factory* factory);
 
+  /* score recognition of the syntax by a block of characters, the
+   *  content identifier or it's suffix or a mime type
+   *  (different from the factory-registered one)
+   */
+  int (*recognise_syntax)(raptor_parser_factory* factory, const unsigned char *buffer, size_t len, const unsigned char *identifier, const unsigned char *suffix, const char *mime_type);
 };
 
 
+/*
+ * Raptor serializer object
+ */
+struct raptor_serializer_s {
+  /* can be filled with error location information */
+  raptor_locator locator;
+
+  void *error_user_data;
+  void *warning_user_data;
+
+  raptor_message_handler error_handler;
+  raptor_message_handler warning_handler;
+
+  /* non 0 if serializer had fatal error and cannot continue */
+  int failed;
+
+  /* base URI of RDF/XML */
+  raptor_uri *base_uri;
+
+  /* serializer specific stuff */
+  void *context;
+
+  /* destination stream for the serialization */
+  raptor_iostream *iostream;
+  
+  struct raptor_serializer_factory_s* factory;
+};
+
+
+/** A Serializer Factory for a syntax */
+struct raptor_serializer_factory_s {
+  struct raptor_serializer_factory_s* next;
+
+  /* syntax name */
+  const char* name;
+  /* alternate syntax name; not mentioned in enumerations */
+  const char* alias;
+
+  /* syntax readable label */
+  const char* label;
+  /* syntax MIME type (or NULL) */
+  const char* mime_type;
+  /* syntax URI (or NULL) */
+  const unsigned char* uri_string;
+  
+  /* the rest of this structure is populated by the
+     serializer-specific register function */
+  size_t context_length;
+  
+  /* create a new serializer */
+  int (*init)(raptor_serializer* serializer, const char *name);
+  
+  /* destroy a serializer */
+  void (*terminate)(raptor_serializer* serializer);
+  
+  /* add a namespace */
+  int (*declare_namespace)(raptor_serializer* serializer, const unsigned char *prefix, raptor_uri *uri);
+  
+  /* start a serialization */
+  int (*serialize_start)(raptor_serializer* serializer);
+  
+  /* serialize a statement */
+  int (*serialize_statement)(raptor_serializer* serializer, const raptor_statement *statment);
+
+  /* end a serialization */
+  int (*serialize_end)(raptor_serializer* serializer);
+  
+  /* finish the serializer factory */
+  void (*finish_factory)(raptor_serializer_factory* factory);
+};
+
+
+/* raptor_serialize.c */
+void raptor_serializer_register_factory(const char *name, const char *label, const char *mime_type, const char *alias, const unsigned char *uri_string, void (*factory) (raptor_serializer_factory*));
 
 
 /* raptor_general.c */
 
-extern const char *raptor_xml_literal_datatype_uri_string;
-
 void raptor_parser_register_factory(const char *name, const char *label, const char *mime_type, const char *alias, const unsigned char *uri_string, void (*factory) (raptor_parser_factory*));
 
 unsigned char* raptor_generate_id(raptor_parser *rdf_parser, const int id_for_bag, unsigned char* user_bnodeid);
@@ -358,10 +516,13 @@ raptor_uri* raptor_inscope_base_uri(raptor_parser *rdf_parser);
 #ifdef RAPTOR_DEBUG
 void raptor_stats_print(raptor_parser *rdf_parser, FILE *stream);
 #endif
+const char* raptor_basename(const char *name);
 
 
 /* raptor_parse.c */
+void raptor_delete_parser_factories(void);
 
+/* raptor_general.c */
 extern void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...);
 extern void raptor_parser_error(raptor_parser* parser, const char *message, ...);
 extern void raptor_parser_simple_error(void* parser, const char *message, ...);
@@ -390,8 +551,6 @@ void raptor_xml_parser_stats_print(raptor_xml_parser* rdf_xml_parser, FILE *stre
 /* raptor_general.c */
 extern void raptor_expat_update_document_locator (raptor_parser *rdf_parser);
 extern int raptor_valid_xml_ID(raptor_parser *rdf_parser, const unsigned char *string);
-char* raptor_vsnprintf(const char *message, va_list arguments);
-void raptor_print_statement_part_as_ntriples(FILE* stream, const void *term, raptor_identifier_type type, raptor_uri* literal_datatype, const unsigned char *literal_language);
 int raptor_check_ordinal(const unsigned char *name);
 
 /* raptor_identifier.c */
@@ -414,6 +573,9 @@ extern void raptor_update_document_locator (raptor_parser *rdf_parser);
 #endif
 
 
+/* raptor_nfc.c */
+int raptor_nfc_check (const unsigned char* string, size_t len, int *error);
+
 /* raptor_nspace.c */
 
 #ifdef RAPTOR_DEBUG
@@ -457,10 +619,23 @@ void raptor_init_parser_ntriples(void);
 void raptor_init_parser_turtle(void);
 void raptor_init_parser_rss(void);
 
+/* raptor_rfc2396.c */
+raptor_uri_detail* raptor_new_uri_detail(const unsigned char *uri_string);
+void raptor_free_uri_detail(raptor_uri_detail* uri_detail);
+unsigned char* raptor_uri_detail_to_string(raptor_uri_detail *ud, size_t* len_p);
+
+/* raptor_serializer.c */
+void raptor_init_serializer_rdfxml(void);
+void raptor_init_serializer_ntriples(void);
+void raptor_init_serializer_simple(void);
+void raptor_delete_serializer_factories(void);
+void raptor_serializer_error(raptor_serializer* serializer, const char *message, ...);
+void raptor_serializer_simple_error(void* serializer, const char *message, ...);
+void raptor_serializer_error_varargs(raptor_serializer* serializer, const char *message,  va_list arguments);
+void raptor_serializer_warning(raptor_serializer* serializer, const char *message, ...);
+void raptor_serializer_warning_varargs(raptor_serializer* serializer, const char *message, va_list arguments);
 
 /* raptor_utf8.c */
-int raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output);
-int raptor_utf8_to_unicode_char(unsigned long *output, const unsigned char *input, int length);
 int raptor_unicode_is_namestartchar(long c);
 int raptor_unicode_is_namechar(long c);
 int raptor_utf8_is_nfc(const unsigned char *input, size_t length);
@@ -479,7 +654,7 @@ int raptor_utf8_is_nfc(const unsigned char *input, size_t length);
 #endif
 
 /* Size of buffer used in various raptor_www places for I/O  */
-#define RAPTOR_WWW_BUFFER_SIZE 256
+#define RAPTOR_WWW_BUFFER_SIZE 4096
 
 /* WWW library state */
 struct  raptor_www_s {
@@ -524,6 +699,8 @@ struct  raptor_www_s {
 
   /* can be filled with error location information */
   raptor_locator locator;
+
+  char *http_accept;
 };
 
 
@@ -549,11 +726,11 @@ void raptor_www_libfetch_free(raptor_www *www);
 int raptor_www_libfetch_fetch(raptor_www *www);
 
   /* raptor_set.c */
-raptor_set* raptor_new_set(void);
-int raptor_free_set(raptor_set* set);
-int raptor_set_add(raptor_set* set, char *item, size_t item_len);
+raptor_id_set* raptor_new_id_set(void);
+void raptor_free_id_set(raptor_id_set* set);
+int raptor_id_set_add(raptor_id_set* set, raptor_uri* base_uri, const unsigned char *item, size_t item_len);
 #ifdef RAPTOR_DEBUG
-void raptor_set_stats_print(raptor_set* set, FILE *stream);
+void raptor_id_set_stats_print(raptor_id_set* set, FILE *stream);
 #endif
 
 /* raptor_sax2.c */
@@ -649,40 +826,28 @@ typedef struct {
 } raptor_triple;
 
 
-/* Sequence class */
-typedef struct raptor_sequence_s raptor_sequence;
-typedef void* (raptor_free_handler(void*));
-typedef void (raptor_print_handler(void *object, FILE *fh));
+/* raptor_rfc2396.c */
+struct raptor_uri_detail_s
+{
+  size_t uri_len;
+  /* buffer is the same size as the original uri_len */
+  unsigned char *buffer;
 
-/* Create */
-RAPTOR_API raptor_sequence* raptor_new_sequence(raptor_free_handler* free_handler, raptor_print_handler* print_handler);
-/* Destroy */
-RAPTOR_API void raptor_free_sequence(raptor_sequence* seq);
-/* Methods */
-RAPTOR_API int raptor_sequence_size(raptor_sequence* seq);
-RAPTOR_API int raptor_sequence_set_at(raptor_sequence* seq, int idx, void *data);
-RAPTOR_API int raptor_sequence_push(raptor_sequence* seq, void *data);
-RAPTOR_API int raptor_sequence_shift(raptor_sequence* seq, void *data);
-RAPTOR_API void* raptor_sequence_get_at(raptor_sequence* seq, int idx);
-RAPTOR_API void* raptor_sequence_pop(raptor_sequence* seq);
-RAPTOR_API void* raptor_sequence_unshift(raptor_sequence* seq);
+  /* URI Components.  These all point into buffer */
+  unsigned char *scheme;
+  unsigned char *authority;
+  unsigned char *path;
+  unsigned char *query;
+  unsigned char *fragment;
 
-RAPTOR_API int raptor_compare_strings(const void *a, const void *b);
-
-RAPTOR_API void raptor_sequence_sort(raptor_sequence* seq, int(*compare)(const void *, const void *));
-
-/* helper for printing sequences of strings */ 
-RAPTOR_API void raptor_sequence_print_string(char *data, FILE *fh);
-RAPTOR_API void raptor_sequence_print(raptor_sequence* seq, FILE* fh);
-
-/* raptor_stringbuffer.c - NOT PUBLIC */
-raptor_stringbuffer* raptor_new_stringbuffer(void);
-void raptor_free_stringbuffer(raptor_stringbuffer *stringbuffer);
-int raptor_stringbuffer_append_counted_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, size_t length, int do_copy);
-int raptor_stringbuffer_append_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, int do_copy);
-int raptor_stringbuffer_append_decimal(raptor_stringbuffer* stringbuffer, int integer);
-unsigned char * raptor_stringbuffer_as_string(raptor_stringbuffer* stringbuffer);
-size_t raptor_stringbuffer_length(raptor_stringbuffer* stringbuffer);
+  /* Lengths of the URI Components  */
+  size_t scheme_len;
+  size_t authority_len;
+  size_t path_len;
+  size_t query_len;
+  size_t fragment_len;
+};
+  
 
 /* end of RAPTOR_INTERNAL */
 #endif
