@@ -2,20 +2,26 @@
  *
  * raptor_libxml.c - Raptor libxml functions
  *
- * $Id: raptor_libxml.c,v 1.19 2003/11/28 19:33:51 cmdjb Exp $
+ * $Id: raptor_libxml.c,v 1.25 2004/09/20 11:54:19 cmdjb Exp $
  *
- * Copyright (C) 2000-2002 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -25,7 +31,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -296,8 +302,8 @@ raptor_libxml_error(void *ctx, const char *msg, ...)
   } else {
     strcpy(nmsg, xml_error_prefix);
     strcpy(nmsg+prefix_length, msg);
-    if(nmsg[length-2]=='\n')
-      nmsg[length-2]='\0';
+    if(nmsg[length-1]=='\n')
+      nmsg[length-1]='\0';
     raptor_parser_error_varargs(rdf_parser, nmsg, args);
     RAPTOR_FREE(cstring,nmsg);
   }
@@ -425,8 +431,8 @@ raptor_libxml_new_entity(raptor_parser* rdf_parser,
 {
   raptor_libxml_entity *ent;
   
-  ent=(raptor_libxml_entity*)RAPTOR_CALLOC(raptor_libxml_entity, 
-                                        sizeof(raptor_libxml_entity), 1);
+  ent=(raptor_libxml_entity*)RAPTOR_CALLOC(raptor_libxml_entity, 1,
+                                           sizeof(raptor_libxml_entity));
   if(!ent) {
     raptor_parser_fatal_error(rdf_parser, "Out of memory");
     return NULL;
@@ -582,5 +588,11 @@ raptor_libxml_init(xmlSAXHandler *sax) {
 }
 
 
+void
+raptor_libxml_free(xmlParserCtxtPtr xc) {
+  libxml2_endDocument(xc);
+  xmlFreeParserCtxt(xc);
+}
+
 /* end if RAPTOR_XML_LIBXML */
 #endif
