@@ -2,20 +2,26 @@
  *
  * raptor_rss.c - Raptor RSS tag soup parser
  *
- * $Id: raptor_rss.c,v 1.17 2004/01/20 12:24:48 cmdjb Exp $
+ * $Id: raptor_rss.c,v 1.26 2004/10/28 21:07:41 cmdjb Exp $
  *
- * Copyright (C) 2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -25,7 +31,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -52,12 +58,16 @@ typedef enum {
   RAPTOR_RSS_IMAGE,
   RAPTOR_RSS_TEXTINPUT,
 
+  /* list items */
+  RAPTOR_RSS_ITEM,
+
+  /* atom author */
+  RAPTOR_ATOM_AUTHOR,
+
   /* also common, but IGNORED */
   RAPTOR_RSS_SKIPHOURS,
   RAPTOR_RSS_SKIPDAYS,
-
-  /* list items */
-  RAPTOR_RSS_ITEM,
+  RAPTOR_RSS_ENCLOSURE,
 
   /* unknown name found */
   RAPTOR_RSS_UNKNOWN,
@@ -73,6 +83,9 @@ typedef enum {
 
 #define RSS1_0_NAMESPACE_URI  "http://purl.org/rss/1.0/"
 #define RSS0_91_NAMESPACE_URI "http://purl.org/rss/1.0/modules/rss091#"
+#define RSS2_0_ENC_NAMESPACE_URI "http://purl.oclc.org/net/rss_2.0/enc#"
+#define ATOM0_3_NAMESPACE_URI "http://purl.org/atom/ns#"
+#define DC_NAMESPACE_URI      "http://purl.org/dc/elements/1.1/"
 
 typedef enum {
   RSS_UNKNOWN_NS = 0,
@@ -81,8 +94,11 @@ typedef enum {
   RSS0_92_NS     = RSS_NO_NS,
   RSS2_0_NS      = RSS_NO_NS,
   RSS1_0_NS      = 4,
+  ATOM0_3_NS     = 5,
+  DC_NS          = 6,
+  RSS2_0_ENC_NS  = 7,
 
-  RSS_NAMESPACES_SIZE = RSS1_0_NS+1
+  RSS_NAMESPACES_SIZE = RSS2_0_ENC_NS+1
 } rss_info_namespace;
 
 
@@ -91,7 +107,10 @@ static const char* const rss_namespace_uri_strings[RSS_NAMESPACES_SIZE]={
   NULL,
   RSS0_91_NAMESPACE_URI,
   NULL,
-  RSS1_0_NAMESPACE_URI
+  RSS1_0_NAMESPACE_URI,
+  ATOM0_3_NAMESPACE_URI,
+  DC_NAMESPACE_URI,
+  RSS2_0_ENC_NAMESPACE_URI
 };
 
 
@@ -105,9 +124,11 @@ raptor_rss_info raptor_rss_types_info[RAPTOR_RSS_COMMON_SIZE]={
   { "channel",    RSS1_0_NS },
   { "image",      RSS1_0_NS },
   { "textinput",  RSS1_0_NS },
+  { "item",       RSS1_0_NS },
+  { "author",     ATOM0_3_NS },
   { "skipHours",  RSS0_91_NS },
   { "skipDays",   RSS0_91_NS },
-  { "item",       RSS1_0_NS },
+  { "Enclosure",  RSS2_0_ENC_NS }, /* Enclosure class in RDF output */
   { "<unknown>",  RSS_UNKNOWN_NS },
 };
 
@@ -138,10 +159,44 @@ typedef enum {
   RAPTOR_RSS_FIELD_AUTHOR,        /* item 2.0 */
   RAPTOR_RSS_FIELD_GUID,          /* item 2.0 */
   RAPTOR_RSS_FIELD_ENCLOSURE,     /* item 0.92, 2.0 */
+  RAPTOR_RSS_RDF_ENCLOSURE,        /* In RDF output, not an RSS field */
+  RAPTOR_RSS_RDF_ENCLOSURE_URL,    /* In RDF output, not an RSS field */
+  RAPTOR_RSS_RDF_ENCLOSURE_LENGTH, /* In RDF output, not an RSS field */
+  RAPTOR_RSS_RDF_ENCLOSURE_TYPE,   /* In RDF output, not an RSS field */
+  RAPTOR_RSS_FIELD_LENGTH,        /* item 0.92, 2.0 */
+  RAPTOR_RSS_FIELD_TYPE,          /* item 0.92, 2.0 */
   RAPTOR_RSS_FIELD_CATEGORY,      /* item 0.92, 2.0, channel 2.0 */
   RAPTOR_RSS_FIELD_COMMENTS,      /* comments v? */
   RAPTOR_RSS_FIELD_ITEMS,         /* rss 1.0 items */
 
+  RAPTOR_RSS_FIELD_ATOM_CONTENT,   /* atom 0.3 content */
+  RAPTOR_RSS_FIELD_ATOM_COPYRIGHT, /* atom 0.3 content */
+  RAPTOR_RSS_FIELD_ATOM_CREATED,   /* atom 0.3 created */
+  RAPTOR_RSS_FIELD_ATOM_ID,        /* atom 0.3 id */
+  RAPTOR_RSS_FIELD_ATOM_ISSUED,    /* atom 0.3 issued */
+  RAPTOR_RSS_FIELD_ATOM_LINK,      /* atom 0.3 link */
+  RAPTOR_RSS_FIELD_ATOM_MODIFIED,  /* atom 0.3 modified */
+  RAPTOR_RSS_FIELD_ATOM_SUMMARY,   /* atom 0.3 summary */
+  RAPTOR_RSS_FIELD_ATOM_TAGLINE,   /* atom 0.3 tagline */
+  RAPTOR_RSS_FIELD_ATOM_TITLE,     /* atom 0.3 title */
+
+  RAPTOR_RSS_FIELD_DC_TITLE,       /* DC title */
+  RAPTOR_RSS_FIELD_DC_CONTRIBUTOR, /* DC contributor */
+  RAPTOR_RSS_FIELD_DC_CREATOR,     /* DC creator */
+  RAPTOR_RSS_FIELD_DC_PUBLISHER,   /* DC publisher */
+  RAPTOR_RSS_FIELD_DC_SUBJECT,     /* DC subject */
+  RAPTOR_RSS_FIELD_DC_DESCRIPTION, /* DC description */
+  RAPTOR_RSS_FIELD_DC_DATE,        /* DC date */
+  RAPTOR_RSS_FIELD_DC_TYPE,        /* DC type */
+  RAPTOR_RSS_FIELD_DC_FORMAT,      /* DC format */
+  RAPTOR_RSS_FIELD_DC_IDENTIFIER,  /* DC identifier */
+  RAPTOR_RSS_FIELD_DC_LANGUAGE,    /* DC language */
+  RAPTOR_RSS_FIELD_DC_RELATION,    /* DC relation */
+  RAPTOR_RSS_FIELD_DC_SOURCE,      /* DC source */
+  RAPTOR_RSS_FIELD_DC_COVERAGE,    /* DC coverage */
+  RAPTOR_RSS_FIELD_DC_RIGHTS,      /* DC rights */
+
+
   RAPTOR_RSS_FIELD_UNKNOWN,
 
   RAPTOR_RSS_FIELD_NONE,
@@ -174,20 +229,87 @@ raptor_rss_info raptor_rss_fields_info[RAPTOR_RSS_FIELDS_SIZE]={
   { "source",         RSS0_92_NS },
   { "author",         RSS2_0_NS },
   { "guid",           RSS2_0_NS },
-  { "enclosure",      RSS0_92_NS },
+  { "enclosure",      RSS2_0_NS },     /* enclosure in RSS */
+  { "enclosure",      RSS2_0_ENC_NS }, /* In RDF output, not an RSS field */
+  { "url",            RSS2_0_ENC_NS }, /* In RDF output, not an RSS field */
+  { "length",         RSS2_0_ENC_NS }, /* In RDF output, not an RSS field */
+  { "type",           RSS2_0_ENC_NS }, /* In RDF output, not an RSS field */
+  { "length",         RSS2_0_NS },
+  { "type",           RSS2_0_NS },
   { "category",       RSS0_92_NS },
   { "comments",       RSS0_92_NS },
   { "items",          RSS1_0_NS },
+
+  { "content",        ATOM0_3_NS },
+  { "copyright",      ATOM0_3_NS },
+  { "created",        ATOM0_3_NS },
+  { "id",             ATOM0_3_NS },
+  { "issued",         ATOM0_3_NS },
+  { "link",           ATOM0_3_NS },
+  { "modified",       ATOM0_3_NS },
+  { "summary",        ATOM0_3_NS },
+  { "tagline",        ATOM0_3_NS },
+  { "title",          ATOM0_3_NS },
+
+  { "title",          DC_NS },
+  { "contributor",    DC_NS },
+  { "creator",        DC_NS },
+  { "publisher",      DC_NS },
+  { "subject",        DC_NS },
+  { "description",    DC_NS },
+  { "date",           DC_NS },
+  { "type",           DC_NS },
+  { "format",         DC_NS },
+  { "identifier",     DC_NS },
+  { "language",       DC_NS },
+  { "relation",       DC_NS },
+  { "source",         DC_NS },
+  { "coverage",       DC_NS },
+  { "rights",         DC_NS },
+
   { "<unknown>",      RSS_UNKNOWN_NS },
 };
 
 
+typedef struct {
+  raptor_rss_fields_type from;
+  raptor_rss_fields_type to;
+} raptor_field_pair;
+
+raptor_field_pair raptor_atom_to_rss[]={
+  { RAPTOR_RSS_FIELD_ATOM_CONTENT,  RAPTOR_RSS_FIELD_DESCRIPTION },
+  { RAPTOR_RSS_FIELD_ATOM_TITLE,    RAPTOR_RSS_FIELD_TITLE },
+#if 0
+  { RAPTOR_RSS_FIELD_ATOM_CREATED,  RAPTOR_RSS_FIELD_UNKNOWN },
+  { RAPTOR_RSS_FIELD_ATOM_ID,       RAPTOR_RSS_FIELD_UNKNOWN },
+  { RAPTOR_RSS_FIELD_ATOM_ISSUED,   RAPTOR_RSS_FIELD_UNKNOWN },
+  { RAPTOR_RSS_FIELD_ATOM_LINK,     RAPTOR_RSS_FIELD_UNKNOWN },
+  { RAPTOR_RSS_FIELD_ATOM_MODIFIED, RAPTOR_RSS_FIELD_UNKNOWN },
+  { RAPTOR_RSS_FIELD_ATOM_SUMMARY,  RAPTOR_RSS_FIELD_UNKNOWN },
+  { RAPTOR_RSS_FIELD_ATOM_TAGLINE,  RAPTOR_RSS_FIELD_UNKNOWN },
+#endif
+  { RAPTOR_RSS_FIELD_UNKNOWN,       RAPTOR_RSS_FIELD_UNKNOWN }
+};
+  
+
+struct raptor_rss_enclosure_s
+{
+  raptor_identifier identifier;
+  raptor_uri *node_type;
+  raptor_uri *url; 
+  char *length;
+  char *type;
+};
+typedef struct raptor_rss_enclosure_s raptor_rss_enclosure;
+
 struct raptor_rss_item_s
 {
   char *uri_string;
   raptor_identifier identifier;
   raptor_uri *node_type;
   char* fields[RAPTOR_RSS_FIELDS_SIZE];
+  raptor_uri* uri_fields[RAPTOR_RSS_FIELDS_SIZE];
+  raptor_rss_enclosure* enclosure;
   int fields_count;
   struct raptor_rss_item_s* next;
 };
@@ -275,6 +397,17 @@ raptor_rss_parse_init(raptor_parser* rdf_parser, const char *name) {
   return 0;
 }
 
+static void
+raptor_enclosure_free(raptor_rss_enclosure* enclosure) {
+  if(enclosure->length)
+    RAPTOR_FREE(cstring, enclosure->length);
+  if(enclosure->type)
+    RAPTOR_FREE(cstring, enclosure->type);
+  if(enclosure->url)
+    raptor_free_uri(enclosure->url);
+  raptor_free_identifier(&(enclosure->identifier));
+  RAPTOR_FREE(raptor_rss_enclosure, enclosure);
+}
 
 static void
 raptor_item_free(raptor_rss_item* item) {
@@ -282,7 +415,11 @@ raptor_item_free(raptor_rss_item* item) {
   for(i=0; i< RAPTOR_RSS_FIELDS_SIZE; i++) {
     if(item->fields[i])
       RAPTOR_FREE(raptor_rss_item, item->fields[i]);
+    if(item->uri_fields[i])
+      raptor_free_uri(item->uri_fields[i]);
   }
+  if(item->enclosure) 
+    raptor_enclosure_free(item->enclosure);
   /* don't free node_type - it is always a shared pointer to a URI */
   raptor_free_identifier(&item->identifier);
 }
@@ -393,10 +530,17 @@ raptor_rss_parse_start(raptor_parser *rdf_parser)
 }
 
 
+static raptor_rss_enclosure*
+raptor_rss_new_enclosure(void)
+{
+  raptor_rss_enclosure* enclosure=(raptor_rss_enclosure*)RAPTOR_CALLOC(raptor_rss_enclosure, 1, sizeof(raptor_rss_enclosure));
+  return enclosure;
+}
+
+
 static void
 raptor_rss_item_add(raptor_rss_parser_context *rss_parser) {
   raptor_rss_item* item=(raptor_rss_item*)RAPTOR_CALLOC(raptor_rss_item, 1, sizeof(raptor_rss_item));
-  
   item->next=NULL;
   
   /* new list */
@@ -419,10 +563,15 @@ static void
 raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
   raptor_rss_parser_context* rss_parser=(raptor_rss_parser_context*)rdf_parser->context;
   xmlTextReaderPtr reader=rss_parser->reader;
-  xmlChar *name, *value;
+  const xmlChar *name;
+  xmlChar *value;
   int type;
+  int is_empty;
+  raptor_uri *uri=NULL;
+  xmlChar *rel=NULL;
+  raptor_rss_enclosure *enclosure=NULL;
   
-  name = xmlTextReaderName(reader);
+  name = xmlTextReaderConstLocalName(reader);
   if (name == NULL)
     name = xmlStrdup(BAD_CAST "--");
   value = xmlTextReaderValue(reader);
@@ -430,12 +579,20 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
   type=xmlTextReaderNodeType(reader);
   
   switch(type) {
-    case 1: /* start element */
+    case XML_READER_TYPE_ELEMENT: /* start element */
+
+      /* Must be checked before moving on to attributes */
+      is_empty=xmlTextReaderIsEmptyElement(reader);
+
       if(rss_parser->current_type==RAPTOR_RSS_NONE) {
         if(!strcmp((const char*)name, "rss") || 
            !strcmp((const char*)name, "rdf") || !strcmp((const char*)name, "RDF")) {
           break;
-        } if(!strcmp((const char*)name, "item")) {
+        } else if(!strcmp((const char*)name, "feed")) {
+          rss_parser->current_type=RAPTOR_RSS_CHANNEL;
+          break;
+        } if(!strcmp((const char*)name, "item") ||
+             !strcmp((const char*)name, "entry")) {
           raptor_rss_item_add(rss_parser);
           rss_parser->current_type=RAPTOR_RSS_ITEM;
         } else {
@@ -458,7 +615,8 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
         raptor_rss_type old_type=rss_parser->current_type;
         
         /* check it is not a type here */
-        if(!strcmp((const char*)name, "item")) {
+        if(!strcmp((const char*)name, "item") ||
+           !strcmp((const char*)name, "entry")) {
           raptor_rss_item_add(rss_parser);
           rss_parser->current_type=RAPTOR_RSS_ITEM;
         } else {
@@ -478,20 +636,51 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
         rss_parser->current_field=RAPTOR_RSS_FIELD_UNKNOWN;
         for(i=0; i<RAPTOR_RSS_FIELDS_SIZE; i++)
           if(!strcmp((const char*)name, raptor_rss_fields_info[i].name)) {
-            rss_parser->current_field=(raptor_rss_fields_type)i;
-            break;
+            const xmlChar *nspace_URI=xmlTextReaderConstNamespaceUri(reader);
+            if(nspace_URI && raptor_rss_fields_info[i].nspace != RSS_NO_NS) {
+              const char *field_nspace_URI=rss_namespace_uri_strings[raptor_rss_fields_info[i].nspace];
+            
+              if(!strcmp((const char*)nspace_URI, field_nspace_URI)) {
+                rss_parser->current_field=(raptor_rss_fields_type)i;
+                break;
+              }
+            } else {
+              rss_parser->current_field=(raptor_rss_fields_type)i;
+              break;
+            }
           }
         
         if(rss_parser->current_field==RAPTOR_RSS_FIELD_UNKNOWN) {
           RAPTOR_DEBUG3("Unknown field element named %s inside type %s\n", name, raptor_rss_types_info[rss_parser->current_type].name);
+        } else if (rss_parser->current_field == RAPTOR_RSS_FIELD_ENCLOSURE ){
+          raptor_rss_item* update_item;
+
+          RAPTOR_DEBUG1("FOUND new enclosure\n");
+          if(rss_parser->current_type == RAPTOR_RSS_ITEM)
+            update_item=rss_parser->last;
+          else
+            update_item=&rss_parser->common[rss_parser->current_type];
+          enclosure=raptor_rss_new_enclosure();
+          update_item->enclosure=enclosure;
         } else {
           RAPTOR_DEBUG4("FOUND field %d - %s inside type %s\n", rss_parser->current_field, raptor_rss_fields_info[rss_parser->current_field].name, raptor_rss_types_info[rss_parser->current_type].name);
+
+          /* Rewrite item fields */
+          for(i=0; raptor_atom_to_rss[i].from != RAPTOR_RSS_FIELD_UNKNOWN; i++) {
+            if(raptor_atom_to_rss[i].from == rss_parser->current_field) {
+              rss_parser->current_field=raptor_atom_to_rss[i].to;
+
+              RAPTOR_DEBUG3("Rewrote into field %d - %s\n", rss_parser->current_field, raptor_rss_fields_info[rss_parser->current_field].name);
+              break;
+            }
+          }
+
         }
       }
 
       /* Now check for attributes */
       while((xmlTextReaderMoveToNextAttribute(reader))) {
-        xmlChar *attrName = xmlTextReaderName(reader);
+        const xmlChar *attrName = xmlTextReaderConstLocalName(reader);
         xmlChar *attrValue = xmlTextReaderValue(reader);
         RAPTOR_DEBUG3("  attribute %s=%s\n", attrName, attrValue);
 
@@ -512,6 +701,9 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
                 attrValue=NULL;
                */
             }
+          } else if (!strcmp((const char*)name, "enclosure") && enclosure) {
+            RAPTOR_DEBUG2("  setting enclosure URL %s\n", attrValue);
+            enclosure->url=raptor_new_uri((const unsigned char*)attrValue);
           }
         } else if(!strcmp((const char*)attrName, "domain")) {
           if(!strcmp((const char*)name, "category")) {
@@ -523,18 +715,74 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
                */
             }
           }
+        } else if(!strcmp((const char*)attrName, "rel")) {
+          rel=attrValue;
+          attrValue=NULL;
+        } else if(!strcmp((const char*)attrName, "href")) {
+          if(!strcmp((const char*)name, "link")) {
+            RAPTOR_DEBUG2("  setting href as URI string for type %s\n", raptor_rss_types_info[rss_parser->current_type].name);
+            if(uri)
+              raptor_free_uri(uri);
+            uri=raptor_new_uri((const unsigned char*)attrValue);
+          }
+        } else if (!strcmp((const char*)attrName, "length")) {
+          if (!strcmp((const char*)name, "enclosure") && enclosure) {
+            size_t len=strlen((const char*)attrValue);
+            RAPTOR_DEBUG2("  setting enclosure length %s\n", attrValue);
+            enclosure->length=(char*)RAPTOR_MALLOC(cstring, len+1);
+            strncpy(enclosure->length, (char*)attrValue, len+1);
+          }
+        } else if (!strcmp((const char*)attrName, "type")) {
+          if (!strcmp((const char*)name, "enclosure") && enclosure) {
+            size_t len=strlen((const char*)attrValue);
+            RAPTOR_DEBUG2("  setting enclosure type %s\n", attrValue);
+            enclosure->type=(char*)RAPTOR_MALLOC(cstring, len+1);
+            strncpy(enclosure->type, (char*)attrValue, len+1);
+          }
         }
-        xmlFree(attrName);
+
         if(attrValue)
           xmlFree(attrValue);
       }
-
-      if(!xmlTextReaderIsEmptyElement(reader))
+      
+      if(!is_empty) {
+        if(uri)
+          raptor_free_uri(uri);
+        if(rel)
+          xmlFree(rel);
         break;
+      }
+
+      /* Empty element, so consider adding one of the attributes as
+       * literal or URI content
+       */
+      if(rss_parser->current_type >= RAPTOR_RSS_COMMON_IGNORED) {
+        /* skipHours, skipDays common but IGNORED */ 
+        RAPTOR_DEBUG3("Ignoring empty element %s for type %s\n", name, raptor_rss_types_info[rss_parser->current_type].name);
+      } else if(uri && rel && !strcmp((const char*)rel, "alternate")) {
+        raptor_rss_item* update_item;
+
+        if(rss_parser->current_type == RAPTOR_RSS_ITEM)
+          update_item=rss_parser->last;
+        else
+          update_item=&rss_parser->common[rss_parser->current_type];
+        
+        RAPTOR_DEBUG3("Added URI to field %s of type %s\n", raptor_rss_fields_info[rss_parser->current_field].name, raptor_rss_types_info[rss_parser->current_type].name);
+        if(!update_item->fields[rss_parser->current_field] &&
+           !update_item->uri_fields[rss_parser->current_field])
+          update_item->fields_count++;
+        update_item->uri_fields[rss_parser->current_field]=uri;
+        uri=NULL;
+      }
+      
+      if(uri)
+        raptor_free_uri(uri);
+      if(rel)
+        xmlFree(rel);
 
       /* FALLTHROUGH if is empty element */
 
-    case 15: /* end element */
+    case XML_READER_TYPE_END_ELEMENT: /* end element */
       if(rss_parser->current_type != RAPTOR_RSS_NONE) {
         if(rss_parser->current_field != RAPTOR_RSS_FIELD_NONE) {
           RAPTOR_DEBUG3("Ending element %s field %s\n", name, raptor_rss_fields_info[rss_parser->current_field].name);
@@ -552,7 +800,10 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
       
       break;
 
-    case 3: /* text */
+    case XML_READER_TYPE_TEXT:
+    case XML_READER_TYPE_SIGNIFICANT_WHITESPACE: /* FIXME */
+    case XML_READER_TYPE_CDATA: /* FIXME */
+
       if((rss_parser->current_type==RAPTOR_RSS_NONE ||
           rss_parser->current_type==RAPTOR_RSS_UNKNOWN) ||
          (rss_parser->current_field==RAPTOR_RSS_FIELD_NONE ||
@@ -564,13 +815,15 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
           p++;
         }
         if(*p)
-          RAPTOR_DEBUG2("IGNORING non-whitespace text node '%s'\n", value);
+          RAPTOR_DEBUG4("IGNORING non-whitespace text '%s' inside type %s, field %s\n", 
+                        value, raptor_rss_types_info[rss_parser->current_type].name,
+                        raptor_rss_fields_info[rss_parser->current_field].name);
         break;
       }
 
-      if(rss_parser->current_type != RAPTOR_RSS_ITEM && 
-         rss_parser->current_type >= RAPTOR_RSS_COMMON_IGNORED) {
+      if(rss_parser->current_type >= RAPTOR_RSS_COMMON_IGNORED) {
         /* skipHours, skipDays common but IGNORED */ 
+        RAPTOR_DEBUG2("Ignoring fields for type %s\n", raptor_rss_types_info[rss_parser->current_type].name);
       } else {
         raptor_rss_item* update_item;
         
@@ -580,7 +833,8 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
           update_item=&rss_parser->common[rss_parser->current_type];
         
         RAPTOR_DEBUG4("Added text '%s' to field %s of type %s\n", value, raptor_rss_fields_info[rss_parser->current_field].name, raptor_rss_types_info[rss_parser->current_type].name);
-        if(!update_item->fields[rss_parser->current_field])
+        if(!update_item->fields[rss_parser->current_field] &&
+           !update_item->uri_fields[rss_parser->current_field])
           update_item->fields_count++;
         update_item->fields[rss_parser->current_field]=(char*)value;
         value=NULL;
@@ -588,15 +842,17 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
       
       break;
 
-    case 4:  /* CData sections */
-    case 5:  /* entity references */
-    case 6:  /* entity declarations */
-    case 7:  /* PIs */
-    case 8:  /* comments */
-    case 9:  /* document nodes */
-    case 10: /* DTD/Doctype nodes */
-    case 11: /* document fragment */
-    case 12: /* notation nodes */
+    case XML_READER_TYPE_ENTITY_REFERENCE:
+    case XML_READER_TYPE_ENTITY:
+    case XML_READER_TYPE_PROCESSING_INSTRUCTION:
+    case XML_READER_TYPE_COMMENT:
+    case XML_READER_TYPE_DOCUMENT:
+    case XML_READER_TYPE_DOCUMENT_TYPE:
+    case XML_READER_TYPE_DOCUMENT_FRAGMENT:
+    case XML_READER_TYPE_NOTATION:
+    case XML_READER_TYPE_WHITESPACE: /* FIXME */
+    case XML_READER_TYPE_END_ENTITY:
+    case XML_READER_TYPE_XML_DECLARATION:
       break;
     
     default:
@@ -613,12 +869,22 @@ raptor_rss_parser_processNode(raptor_parser *rdf_parser) {
       RAPTOR_DEBUG2("Ignoring type %d\n", type);
   }
     
-  xmlFree(name);
   if(value)
     xmlFree(value);
 
 }
 
+static void
+raptor_rss_insert_enclosure_identifiers(raptor_parser* rdf_parser, 
+                                        raptor_rss_enclosure *enclosure)
+{
+  raptor_identifier* identifier=&enclosure->identifier;
+
+  identifier->id=raptor_generate_id(rdf_parser, 0, NULL);
+  identifier->type=RAPTOR_IDENTIFIER_TYPE_ANONYMOUS;
+  identifier->uri_source=RAPTOR_URI_SOURCE_GENERATED;
+  enclosure->node_type=raptor_rss_types_info[RAPTOR_RSS_ENCLOSURE].uri;
+}
 
 static void
 raptor_rss_insert_identifiers(raptor_parser* rdf_parser) 
@@ -637,17 +903,23 @@ raptor_rss_insert_identifiers(raptor_parser* rdf_parser)
       continue;
     
     if(item->uri_string) {
-      identifier->uri=raptor_new_uri((const unsigned char*)item->fields[RAPTOR_RSS_FIELD_LINK]);
+      identifier->uri=raptor_new_uri((const unsigned char*)item->uri_string);
       identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
       identifier->uri_source=RAPTOR_URI_SOURCE_URI;
     } else {
       int url_field=(i== RAPTOR_RSS_IMAGE) ? RAPTOR_RSS_FIELD_URL :
                                              RAPTOR_RSS_FIELD_LINK;
+      if(i == RAPTOR_RSS_CHANNEL)
+        url_field=RAPTOR_RSS_FIELD_ATOM_ID;
       
       if(item->fields[url_field]) {
         identifier->uri=raptor_new_uri((const unsigned char*)item->fields[url_field]);
         identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
         identifier->uri_source=RAPTOR_URI_SOURCE_URI;
+      } else if(item->uri_fields[url_field]) {
+        identifier->uri=raptor_uri_copy(item->uri_fields[url_field]);
+        identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+        identifier->uri_source=RAPTOR_URI_SOURCE_URI;
       } else {
         /* need to make bnode */
         identifier->id=raptor_generate_id(rdf_parser, 0, NULL);
@@ -655,6 +927,10 @@ raptor_rss_insert_identifiers(raptor_parser* rdf_parser)
         identifier->uri_source=RAPTOR_URI_SOURCE_GENERATED;
       }
     }
+
+    if(item->enclosure)
+      raptor_rss_insert_enclosure_identifiers(rdf_parser, item->enclosure);
+
     item->node_type=raptor_rss_types_info[i].uri;
   }
 
@@ -663,7 +939,7 @@ raptor_rss_insert_identifiers(raptor_parser* rdf_parser)
     raptor_identifier* identifier=&item->identifier;
     
     if(item->uri_string) {
-      identifier->uri=raptor_new_uri((const unsigned char*)item->fields[RAPTOR_RSS_FIELD_LINK]);
+      identifier->uri=raptor_new_uri((const unsigned char*)item->uri_string);
       identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
       identifier->uri_source=RAPTOR_URI_SOURCE_URI;
     } else {
@@ -671,6 +947,18 @@ raptor_rss_insert_identifiers(raptor_parser* rdf_parser)
         identifier->uri=raptor_new_uri((const unsigned char*)item->fields[RAPTOR_RSS_FIELD_LINK]);
         identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
         identifier->uri_source=RAPTOR_URI_SOURCE_URI;
+      } else if(item->uri_fields[RAPTOR_RSS_FIELD_LINK]) {
+        identifier->uri=raptor_uri_copy(item->uri_fields[RAPTOR_RSS_FIELD_LINK]);
+        identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+        identifier->uri_source=RAPTOR_URI_SOURCE_URI;
+      } else if(item->fields[RAPTOR_RSS_FIELD_ATOM_LINK]) {
+        identifier->uri=raptor_new_uri((const unsigned char*)item->fields[RAPTOR_RSS_FIELD_ATOM_LINK]);
+        identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+        identifier->uri_source=RAPTOR_URI_SOURCE_URI;
+      } else if(item->uri_fields[RAPTOR_RSS_FIELD_ATOM_LINK]) {
+        identifier->uri=raptor_uri_copy(item->uri_fields[RAPTOR_RSS_FIELD_ATOM_LINK]);
+        identifier->type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+        identifier->uri_source=RAPTOR_URI_SOURCE_URI;
       } else {
         /* need to make bnode */
         identifier->id=raptor_generate_id(rdf_parser, 0, NULL);
@@ -678,6 +966,10 @@ raptor_rss_insert_identifiers(raptor_parser* rdf_parser)
         identifier->uri_source=RAPTOR_URI_SOURCE_GENERATED;
       }
     }
+
+    if(item->enclosure)
+      raptor_rss_insert_enclosure_identifiers(rdf_parser, item->enclosure);
+
     item->node_type=raptor_rss_types_info[RAPTOR_RSS_ITEM].uri;
   }
 
@@ -705,6 +997,43 @@ raptor_rss_emit_type_triple(raptor_parser* rdf_parser,
 
 }
 
+static void
+raptor_rss_emit_enclosure(raptor_parser* rdf_parser, 
+                          raptor_rss_enclosure *enclosure)
+{
+  raptor_rss_parser_context* rss_parser=(raptor_rss_parser_context*)rdf_parser->context;
+  raptor_identifier* identifier=&enclosure->identifier;
+
+  rss_parser->statement.predicate=raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE].uri;
+  rss_parser->statement.object=identifier->id;
+  rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_ANONYMOUS;
+  (*rdf_parser->statement_handler)(rdf_parser->user_data, &rss_parser->statement);
+
+  raptor_rss_emit_type_triple(rdf_parser, identifier, enclosure->node_type);
+
+  if (enclosure->url) {
+    rss_parser->statement.predicate=raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_URL].uri;
+    rss_parser->statement.object=enclosure->url;
+    rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+    (*rdf_parser->statement_handler)(rdf_parser->user_data, &rss_parser->statement);
+  }
+
+  if (enclosure->type) {
+    rss_parser->statement.predicate=raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_TYPE].uri;
+    rss_parser->statement.object=enclosure->type;
+    rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_LITERAL;
+    (*rdf_parser->statement_handler)(rdf_parser->user_data, &rss_parser->statement);
+  }
+
+  if (enclosure->length) {
+    rss_parser->statement.predicate=raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH].uri;
+    rss_parser->statement.object=enclosure->length;
+    rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_LITERAL;
+    (*rdf_parser->statement_handler)(rdf_parser->user_data, &rss_parser->statement);
+  }
+
+  rss_parser->statement.subject=enclosure->identifier.id;
+}
 
 
 static void
@@ -718,22 +1047,29 @@ raptor_rss_emit_item(raptor_parser* rdf_parser, raptor_rss_item *item)
     return;
 
   raptor_rss_emit_type_triple(rdf_parser, identifier, item->node_type);
-  
-  for(f=0; f< RAPTOR_RSS_COMMON_SIZE; f++) {
-    char* value=item->fields[f];
-    if(!value)
+
+  for(f=0; f< RAPTOR_RSS_FIELDS_SIZE; f++) {
+    if(!item->fields[f] && !item->uri_fields[f])
       continue;
     
     rss_parser->statement.predicate=raptor_rss_fields_info[f].uri;
     if(!rss_parser->statement.predicate)
       continue;
     
-    rss_parser->statement.object=value;
-    rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_LITERAL;
+    if(item->fields[f]) {
+      rss_parser->statement.object=item->fields[f];
+      rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_LITERAL;
+    } else {
+      rss_parser->statement.object=item->uri_fields[f];
+      rss_parser->statement.object_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+    }
     
     /* Generate the statement */
     (*rdf_parser->statement_handler)(rdf_parser->user_data, &rss_parser->statement);
   }
+
+  if(item->enclosure)
+    raptor_rss_emit_enclosure(rdf_parser, item->enclosure);
 }
 
 
@@ -809,7 +1145,6 @@ raptor_rss_emit(raptor_parser* rdf_parser)
                                raptor_rss_fields_info[RAPTOR_RSS_FIELD_ITEMS].uri, 0,
                                items);
     
-    
     /* sequence of rss:item */
     for(i=1, item=rss_parser->items; item; item=item->next, i++) {
       
@@ -877,6 +1212,32 @@ raptor_rss_parse_chunk(raptor_parser* rdf_parser,
   return (ret != 0);
 }
 
+
+static int
+raptor_rss_parse_recognise_syntax(raptor_parser_factory* factory, 
+                                  const unsigned char *buffer, size_t len,
+                                  const unsigned char *identifier, 
+                                  const unsigned char *suffix, 
+                                  const char *mime_type)
+{
+  int score= 0;
+  
+  if(suffix) {
+    if(!strcmp((const char*)suffix, "rss"))
+      score=7;
+  }
+  
+  if(identifier) {
+    if(strstr((const char*)identifier, "rss2"))
+      score+=5;
+    else if(!suffix && strstr((const char*)identifier, "rss"))
+      score+=4;
+  }
+  
+  return score;
+}
+
+
 static void
 raptor_rss_parser_register_factory(raptor_parser_factory *factory) 
 {
@@ -886,6 +1247,7 @@ raptor_rss_parser_register_factory(raptor_parser_factory *factory)
   factory->terminate = raptor_rss_parse_terminate;
   factory->start     = raptor_rss_parse_start;
   factory->chunk     = raptor_rss_parse_chunk;
+  factory->recognise_syntax = raptor_rss_parse_recognise_syntax;
 }
 
 
