@@ -2,20 +2,26 @@
  *
  * raptor_utf8.c - Raptor UTF-8 and Unicode support
  *
- * $Id: raptor_utf8.c,v 1.10 2004/01/19 00:01:47 cmdjb Exp $
+ * $Id: raptor_utf8.c,v 1.16 2004/09/20 11:54:22 cmdjb Exp $
  *
- * Copyright (C) 2002 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -25,7 +31,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -34,15 +40,12 @@
 #include <stdlib.h>
 #endif
 
-#ifdef RAPTOR_NFC_CHECK
-#ifdef HAVE_G_UTF8_NORMALIZE
-#include <glib.h>
-#endif
-#endif
-
 /* Raptor includes */
 #include "raptor.h"
 #include "raptor_internal.h"
+#ifdef RAPTOR_NFC_CHECK
+#include "raptor_nfc.h"
+#endif
 
 
 /*
@@ -74,37 +77,37 @@ raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output)
 
   switch(size) {
     case 6:
-      output[5]=0x80 | (c & 0x3F);
+      output[5]=0x80 | (unsigned char)(c & 0x3F);
       c= c >> 6;
        /* set bit 2 (bits 7,6,5,4,3,2 less 7,6,5,4,3 set below) on last byte */
       c |= 0x4000000; /* 0x10000 = 0x04 << 24 */
       /* FALLTHROUGH */
     case 5:
-      output[4]=0x80 | (c & 0x3F);
+      output[4]=0x80 | (unsigned char)(c & 0x3F);
       c= c >> 6;
        /* set bit 3 (bits 7,6,5,4,3 less 7,6,5,4 set below) on last byte */
       c |= 0x200000; /* 0x10000 = 0x08 << 18 */
       /* FALLTHROUGH */
     case 4:
-      output[3]=0x80 | (c & 0x3F);
+      output[3]=0x80 | (unsigned char)(c & 0x3F);
       c= c >> 6;
        /* set bit 4 (bits 7,6,5,4 less 7,6,5 set below) on last byte */
       c |= 0x10000; /* 0x10000 = 0x10 << 12 */
       /* FALLTHROUGH */
     case 3:
-      output[2]=0x80 | (c & 0x3F);
+      output[2]=0x80 | (unsigned char)(c & 0x3F);
       c= c >> 6;
       /* set bit 5 (bits 7,6,5 less 7,6 set below) on last byte */
       c |= 0x800; /* 0x800 = 0x20 << 6 */
       /* FALLTHROUGH */
     case 2:
-      output[1]=0x80 | (c & 0x3F);
+      output[1]=0x80 | (unsigned char)(c & 0x3F);
       c= c >> 6;
       /* set bits 7,6 on last byte */
       c |= 0xc0; 
       /* FALLTHROUGH */
     case 1:
-      output[0]=c;
+      output[0]=(unsigned char)c;
   }
 
   return size;
@@ -684,9 +687,6 @@ raptor_utf8_is_nfc(const unsigned char *input, size_t length)
 {
   unsigned int i;
   int plain=1;
-#if defined(RAPTOR_NFC_CHECK) && defined (HAVE_G_UTF8_NORMALIZE)
-  unsigned char *norm;
-#endif
   
   for(i=0; i<length; i++)
     if(input[i]>0x7f) {
@@ -696,17 +696,10 @@ raptor_utf8_is_nfc(const unsigned char *input, size_t length)
     
   if(plain)
     return 1;
-  
-#if defined(RAPTOR_NFC_CHECK) && defined(HAVE_G_UTF8_NORMALIZE)
-  norm=(unsigned char*)g_utf8_normalize((const gchar*)input, length, G_NORMALIZE_NFKC);
-  for(i=0; i<length; i++)
-    if(input[i] != norm[i]) {
-      SYSTEM_FREE(norm);
-      return 0;
-    }
-  
-  SYSTEM_FREE(norm);
-#endif
 
+#ifdef RAPTOR_NFC_CHECK  
+  return raptor_nfc_check(input, length, NULL);
+#else
   return 1;
+#endif
 }
