@@ -2,20 +2,25 @@
  *
  * rdfproc.c - Redland RDF processor
  *
- * $Id: rdfproc.c,v 1.35 2004/01/20 22:18:04 cmdjb Exp $
+ * $Id: rdfproc.c,v 1.50 2004/11/01 20:43:35 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
@@ -76,6 +81,7 @@ enum command_type {
   CMD_ARCS_OUT,
   CMD_HAS_ARC_IN,
   CMD_HAS_ARC_OUT,
+  CMD_QUERY_AS_BINDINGS,
   CMD_QUERY,
   CMD_SERIALIZE,
   CMD_REMOVE_CONTEXT,
@@ -113,7 +119,8 @@ static command commands[]={
   {CMD_HAS_ARC_OUT, "has-arc-out", 2, 2, 0},
   /* FIXME triples-match-query is deliberately not documented */
   {CMD_QUERY, "triples-match-query", 3, 3, 0},
-  {CMD_SERIALIZE, "serialize", 0, 3, 0},
+  {CMD_QUERY_AS_BINDINGS, "query", 3, 3, 0},
+  {CMD_SERIALIZE, "serialize", 0, 4, 0},
   {CMD_REMOVE_CONTEXT, "remove-context", 1, 1, 0},
   {CMD_CONTEXTS, "contexts", 0, 0, 0},
   {CMD_MATCH, "match", 3, 4, 0},
@@ -154,6 +161,37 @@ static const char *default_storage_name="hashes";
 static const char *default_storage_options="hash-type='bdb',dir='.'";
 
 
+static int
+log_handler(void *user_data, librdf_log_message *message) 
+{
+  /* int code=message->code; */ /* The error code */
+  raptor_locator *locator=(raptor_locator*)(message->locator);
+
+  /* Do not handle messages below warning*/
+  if(message->level < LIBRDF_LOG_WARN)
+    return 0;
+
+  if(message->level == LIBRDF_LOG_WARN)
+    fprintf(stderr, "%s: Warning - ", program);
+  else
+    fprintf(stderr, "%s: Error - ", program);
+
+  if(locator) { /* && message->facility == LIBRDF_FROM_PARSER) */
+    raptor_print_locator(stderr, locator); 
+    fputc(':', stderr);
+    fputc(' ', stderr);
+ }
+  fputs(message->message, stderr);
+  fputc('\n', stderr);
+  if(message->level >= LIBRDF_LOG_FATAL)
+    exit(1);
+
+  /* Handled */
+  return 1;
+}
+
+  
+
 int
 main(int argc, char *argv[]) 
 {
@@ -164,16 +202,17 @@ main(int argc, char *argv[])
   librdf_model* model;
   librdf_statement* partial_statement;
   librdf_node *source, *arc, *target, *node;
+  librdf_node* context_node=NULL;
   librdf_stream* stream;
   librdf_iterator* iterator;
   librdf_uri *uri;
   librdf_uri *base_uri;
   librdf_query *query;
+  librdf_query_results *results;
   librdf_hash *options;
   int usage=0;
   int help=0;
   int is_new=0;
-  int contexts=0;
   char *identifier=NULL;
   char *cmd=NULL;
   char *name;
@@ -204,6 +243,9 @@ main(int argc, char *argv[])
   argv[0]=program;
 
   world=librdf_new_world();
+
+  librdf_world_set_logger(world, NULL, log_handler);
+
   librdf_world_open(world);
 
   options=librdf_new_hash(world, NULL);
@@ -430,9 +472,9 @@ main(int argc, char *argv[])
     puts("      with optional BASEURI, into the optional CONTEXT.");
     puts("  print                                     Print the graph triples.");
     puts("  serialize [SYNTAX [URI [MIME-TYPE]]]      Serializes to a syntax (RDF/XML).");
+    puts("  query NAME|- URI|- QUERY-STRING           Run QUERY-STRING query in language NAME for bindings");
 #if 0
-    /* FIXME Not a query -> results table query, so leave out for now */
-    puts("  query NAME URI|- QUERY-STRING             Query for matching triples");
+    puts("  triples-match-query NAME URI|- QUERY-STRING  Query for matching triples");
 #endif
     puts("  find SUBJECT|- PREDICATE|- OBJECT|- [CONTEXT] Find matching triples");
     puts("  contains SUBJECT PREDICATE OBJECT         Check if triple is in the graph.");
@@ -451,12 +493,12 @@ main(int argc, char *argv[])
     puts("    URIs like http://example.org otherwise are literal strings.");
     puts("    - matches any node when allowed.");
     puts("  triples are three nodes (subject, predicate, object)");
-    puts("    predicates cannot be blank node identifiers, only objects can be literals");
+    puts("    predicates can only be uris, only objects can be literals");
     puts("  source means subject of triples matching (?,  NODE1, NODE2)");
     puts("  target means object of triples matching (NODE1, NODE2, ?)");
     puts("  arc means predicate of triples matching (NODE1, ?, NODE2)");
     puts("\nReport bugs to <redland-dev@lists.librdf.org>.");
-    puts("Redland home page: http://www.redland.opensource.ac.uk/");
+    puts("Redland home page: http://librdf.org/");
     exit(0);
   }
   
@@ -487,18 +529,6 @@ main(int argc, char *argv[])
   }
 
 
-  uri=librdf_new_uri(world, (const unsigned char*)LIBRDF_MODEL_FEATURE_CONTEXTS);
-  contexts=0;
-  if(uri) {
-    node=librdf_model_get_feature(model, uri);
-    if(node) {
-      contexts=atoi((const char*)librdf_node_get_literal_value(node));
-      librdf_free_node(node);
-    }
-    librdf_free_uri(uri);
-  }
-
-
   /* Do this or gcc moans */
   stream=NULL;
   iterator=NULL;
@@ -510,6 +540,7 @@ main(int argc, char *argv[])
   uri=NULL;
   node=NULL;
   query=NULL;
+  results=NULL;
 
   switch(type) {
     case CMD_PRINT:
@@ -640,11 +671,12 @@ main(int argc, char *argv[])
       break;
 
     case CMD_SERIALIZE:
-        /* args are name (optional), uri (may be NULL), mime_type (optional) */
+        /* args are name (optional), uri (may be NULL), mime_type (optional), base URI (optional) */
 
         uri=NULL;
         name=NULL;
         mime_type=NULL;
+        base_uri=NULL;
 
         if(!argc)
           goto serialize;
@@ -668,6 +700,16 @@ main(int argc, char *argv[])
             mime_type=argv[2];
         }
 
+	if(argc ==4) {
+          if(strcmp(argv[3], "-")) {
+            base_uri=librdf_new_uri(world, (const unsigned char *)argv[3]);
+            if(!base_uri) {
+              fprintf(stderr, "%s: Failed to create base URI from %s\n", program, argv[3]);
+              break;
+            }
+          }
+        }
+
       serialize:
         serializer=librdf_new_serializer(world, name, mime_type, uri);
         if(!serializer) {
@@ -677,7 +719,7 @@ main(int argc, char *argv[])
           break;
         }
 
-        librdf_serializer_serialize_model(serializer, stdout, NULL, model);
+        librdf_serializer_serialize_model_to_file_handle(serializer, stdout, NULL, model);
 
         librdf_free_serializer(serializer);
         if(uri)
@@ -688,6 +730,32 @@ main(int argc, char *argv[])
     case CMD_QUERY:
       /* args are name, uri (may be NULL), query_string/mime_type */
 
+      name=argv[0];
+      if(!strcmp(name, "-"))
+        name=NULL;
+      
+      if(!strcmp(argv[1], "-"))
+        uri=NULL;
+      else {
+        uri=librdf_new_uri(world, (const unsigned char *)argv[1]);
+        if(!uri) {
+          fprintf(stderr, "%s: Failed to create URI from %s\n", program, argv[1]);
+          break;
+        }
+      }
+
+      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2]);
+      goto printmatching;
+      break;
+
+    case CMD_QUERY_AS_BINDINGS:
+      /* args are name (optional), uri (may be NULL), query_string */
+      uri=NULL;
+      name=NULL;
+      
+      if(strcmp(argv[0], "-"))
+        name=argv[0];
+
       if(!strcmp(argv[1], "-"))
         uri=NULL;
       else {
@@ -698,10 +766,51 @@ main(int argc, char *argv[])
         }
       }
 
-      query=librdf_new_query(world, argv[0], uri, (const unsigned char *)argv[2]);
-      goto printmatching;
-      break;
+      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2]);
+      if(!query) {
+        fprintf(stderr, "%s: Failed to create new query %s\n", program, argv[2]);
+        if(uri)
+          librdf_free_uri(uri);
+        break;
+      }
 
+      if(!(results=librdf_model_query_execute(model, query))) {
+        fprintf(stderr, "%s: Query of model with '%s' failed\n", 
+                program, argv[2]);
+        if(uri)
+          librdf_free_uri(uri);
+        librdf_free_query(query);
+        query=NULL;
+        break;
+      }
+
+      while(!librdf_query_results_finished(results)) {
+        fputs("result: [", stdout);
+        for(i=0; i<librdf_query_results_get_bindings_count(results); i++) {
+          librdf_node *value=librdf_query_results_get_binding_value(results, i);
+          name=(char*)librdf_query_results_get_binding_name(results, i);
+
+          if(i>0)
+            fputs(", ", stdout);
+          fprintf(stdout, "%s=", name);
+          if(value) {
+            librdf_node_print(value, stdout);
+            librdf_free_node(value);
+          } else
+            fputs("NULL", stdout);
+        }
+        fputs("]\n", stdout);
+        
+        librdf_query_results_next(results);
+      }
+
+      fprintf(stdout, "%s: Query returned %d results\n", program, 
+              librdf_query_results_get_count(results));
+      if(uri)
+        librdf_free_uri(uri);
+      librdf_free_query_results(results);
+      break;
+      
     case CMD_CONTAINS:
     case CMD_FIND:
     case CMD_MATCH:
@@ -767,8 +876,6 @@ main(int argc, char *argv[])
           /* Print out matching statements */
           if(type==CMD_FIND || type==CMD_MATCH) {
             if(argc == 4 || type==CMD_MATCH) {
-              librdf_node *context_node=NULL;
-              
               if(argc == 4) {
                 if (librdf_heuristic_is_blank_node(argv[3]))
                   context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
@@ -789,8 +896,16 @@ main(int argc, char *argv[])
                 librdf_free_node(context_node);
             } else
               stream=librdf_model_find_statements(model, partial_statement);
-          } else
-            stream=librdf_model_query(model, query);
+          } else {
+            if(!(results=librdf_model_query_execute(model, query))) {
+              fprintf(stderr, "%s: Query of model with '%s' failed\n", 
+                      program, argv[2]);
+              librdf_free_query(query);
+              query=NULL;
+              break;
+            }
+            stream=librdf_query_results_as_stream(results);
+          }
 
           if(!stream) {
             fprintf(stderr, "%s: %s returned no results (NULL stream)\n", program, commands[type].name);
@@ -798,7 +913,7 @@ main(int argc, char *argv[])
             count=0;
             while(!librdf_stream_end(stream)) {
               librdf_statement *statement=librdf_stream_get_object(stream);
-              librdf_node *context_node=(librdf_node*)librdf_stream_get_context(stream);
+              context_node=(librdf_node*)librdf_stream_get_context(stream);
               if(!statement) {
                 fprintf(stderr, "%s: librdf_stream_next returned NULL\n", program);
                 break;
@@ -824,14 +939,15 @@ main(int argc, char *argv[])
 
             if(!output_model)
               fprintf(stderr, "%s: matching triples: %d\n", program, count);
+
+            if(results)
+              librdf_free_query_results(results);
           }
           break;
           
         case CMD_ADD:
         case CMD_ADD_TYPED:
-          if(argv[3] && contexts) {
-            librdf_node* context_node;
-
+          if(argv[3]) {
             if (librdf_heuristic_is_blank_node(argv[3]))
               context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
             else
@@ -852,9 +968,7 @@ main(int argc, char *argv[])
           break;
           
         case CMD_REMOVE:
-          if(argv[3] && contexts) {
-            librdf_node* context_node;
-
+          if(argv[3]) {
             if (librdf_heuristic_is_blank_node(argv[3]))
               context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
             else
@@ -863,11 +977,8 @@ main(int argc, char *argv[])
             rc=librdf_model_context_remove_statement(model, context_node,
                                                      partial_statement);
             librdf_free_node(context_node);
-          } else {
-            if(argv[3])
-              fprintf(stderr, "%s: ERROR: Cannot remove triple in context - model does not support contexts\n", program);
+          } else
             rc=librdf_model_remove_statement(model, partial_statement);
-          }
           if(rc)
             fprintf(stdout, "%s: failed to remove triple from the graph\n", program);
           else
@@ -936,7 +1047,7 @@ main(int argc, char *argv[])
       /* (Common code) Print out nodes */
       count=0;
       while(!librdf_iterator_end(iterator)) {
-        librdf_node *context_node=(librdf_node*)librdf_iterator_get_context(iterator);
+        context_node=(librdf_node*)librdf_iterator_get_context(iterator);
         node=(librdf_node*)librdf_iterator_get_object(iterator);
         if(!node) {
           fprintf(stderr, "%s: librdf_iterator_get_object returned NULL\n",
@@ -1056,7 +1167,7 @@ main(int argc, char *argv[])
 
       count=0;
       while(!librdf_iterator_end(iterator)) {
-        librdf_node *context_node=(librdf_node*)librdf_iterator_get_context(iterator);
+        context_node=(librdf_node*)librdf_iterator_get_context(iterator);
         node=(librdf_node*)librdf_iterator_get_object(iterator);
         if(!node) {
           fprintf(stderr, "%s: librdf_iterator_get_next returned NULL\n",
@@ -1102,56 +1213,47 @@ main(int argc, char *argv[])
       break;
 
     case CMD_REMOVE_CONTEXT:
-      if(contexts) {
-        librdf_node* context_node;
-
-        if (librdf_heuristic_is_blank_node(argv[0]))
-          context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
-        else
-          context_node=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
+      if (librdf_heuristic_is_blank_node(argv[0]))
+        context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
+      else
+        context_node=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
       
-        rc=librdf_model_context_remove_statements(model, context_node);
-        librdf_free_node(context_node);
-        
-        if(rc)
-          fprintf(stdout, "%s: failed to remove context triples from the graph\n", program);
-        else
-          fprintf(stdout, "%s: removed context triples from the graph\n", program);
-      } else
-        fprintf(stderr, "%s: ERROR: Cannot remove contexts - model does not support contexts\n", program);
-
+      rc=librdf_model_context_remove_statements(model, context_node);
+      librdf_free_node(context_node);
+      
+      if(rc)
+        fprintf(stdout, "%s: failed to remove context triples from the graph\n", program);
+      else
+        fprintf(stdout, "%s: removed context triples from the graph\n", program);
       break;
 
 
     case CMD_CONTEXTS:
-      if(contexts) {
-        iterator=librdf_model_get_contexts(model);
-        if(!iterator) {
-          fprintf(stderr, "%s: Failed to get contexts\n", program);
+      iterator=librdf_model_get_contexts(model);
+      if(!iterator) {
+        fprintf(stderr, "%s: Failed to get contexts\n", program);
+        break;
+      }
+      
+      count=0;
+      while(!librdf_iterator_end(iterator)) {
+        node=(librdf_node*)librdf_iterator_get_object(iterator);
+        if(!node) {
+          fprintf(stderr, "%s: librdf_iterator_get_next returned NULL\n",
+                  program);
           break;
         }
-
-        count=0;
-        while(!librdf_iterator_end(iterator)) {
-          node=(librdf_node*)librdf_iterator_get_object(iterator);
-          if(!node) {
-            fprintf(stderr, "%s: librdf_iterator_get_next returned NULL\n",
-                    program);
-            break;
-          }
-          
-          fputs("Context: ", stdout);
-          librdf_node_print(node, stdout);
-          fputc('\n', stdout);
-          
-          librdf_free_node(node);
-          count++;
-          librdf_iterator_next(iterator);
-        }
-        librdf_free_iterator(iterator);
-        fprintf(stderr, "%s: contexts: %d\n", program, count);
-} else
-        fprintf(stderr, "%s: ERROR: Cannot list contexts - model does not support contexts\n", program);
+        
+        fputs("Context: ", stdout);
+        librdf_node_print(node, stdout);
+        fputc('\n', stdout);
+        
+        librdf_free_node(node);
+        count++;
+        librdf_iterator_next(iterator);
+      }
+      librdf_free_iterator(iterator);
+      fprintf(stderr, "%s: contexts: %d\n", program, count);
       break;
 
 
@@ -1167,8 +1269,9 @@ main(int argc, char *argv[])
     SYSTEM_FREE(uri_string);
 
   if(output_model) {
-    if(librdf_serializer_serialize_model(output_serializer, stdout, NULL,
-                                         output_model)) {
+    if(librdf_serializer_serialize_model_to_file_handle(output_serializer, 
+                                                        stdout, NULL,
+                                                        output_model)) {
       fprintf(stderr, "%s: Failed to serialize output model\n", program);
     };
     librdf_free_serializer(output_serializer);
